@@ -18,25 +18,28 @@
  */
 
 #include <config.h>
-#include <ctype.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <inttypes.h>
-#include "xvasprintf.h"
-#include "xstrndup.h"
+#include <ctype.h>		/* C89 */
+#include <string.h>		/* ? */
+#include <sys/types.h>		/* ? */
+#include <sys/stat.h>		/* ? */
+#include <unistd.h>		/* POSIX */
+#include <netdb.h>		/* ? */
+#include <sys/socket.h>		/* ? */
+#include <netinet/in.h>		/* ? */
+#include <arpa/inet.h>		/* ? */
+#include <inttypes.h>		/* ? */
+#include "xvasprintf.h"		/* Gnulib */
+#include "xstrndup.h"		/* Gnulib */
+#include "xalloc.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/error.h"
-#include "microdc.h"
 #include "common/strleftcmp.h"
-#include "common/memory.h"
 #include "common/intparse.h"
 #include "common/common.h"
 #include "common/strbuf.h"
+#include "microdc.h"
 
 static void cmd_exit(char *cmd, char *args);
 static void cmd_nick(char *cmd, char *args);
@@ -68,6 +71,7 @@ static void cmd_tag(char *cmd, char *args);
 static void cmd_transfers(char *cmd, char *args);
 static void cmd_cancel(char *cmd, char *args);
 static void cmd_debug(char *cmd, char *args);
+static void cmd_listenaddr(char *cmd, char *args);
 static void cmd_listenport(char *cmd, char *args);
 static void cmd_search(char *cmd, char *args);
 static void cmd_results(char *cmd, char *args);
@@ -83,7 +87,7 @@ struct DCCommand {
 /* This structure must be sorted by command name, according to strcmp. */
 static struct DCCommand commands[] = {
     { "active",     	cmd_active, 	    0, NULL },
-    { "browse",     	cmd_browse, 	    DC_CPL_SIMPLE, user_completion_generator },
+    { "browse",     	cmd_browse, 	    DC_CPL_SIMPLE, user_or_myself_completion_generator },
     { "cancel",         cmd_cancel,         DC_CPL_SIMPLE, transfer_completion_generator },
     { "cd", 	    	cmd_cd,     	    DC_CPL_FILE, remote_dir_completion_generator },
     { "connect",    	cmd_connect, 	    0, NULL },
@@ -96,6 +100,7 @@ static struct DCCommand commands[] = {
     { "find",		cmd_find,	    DC_CPL_FILE, remote_path_completion_generator },
     { "get", 	    	cmd_get,    	    DC_CPL_FILE, remote_path_completion_generator },
     { "grantslot",  	cmd_grantslot,      DC_CPL_SIMPLE, user_completion_generator },
+    { "listenaddr",	cmd_listenaddr,     0, NULL },
     { "listenport",  	cmd_listenport,     0, NULL },
     { "listingdir", 	cmd_listingdir,     DC_CPL_FILE, local_dir_completion_generator },
     { "ls", 	    	cmd_ls,     	    DC_CPL_FILE, remote_path_completion_generator },
@@ -264,7 +269,7 @@ command_execute(char *line)
     if (i >= 0) {
 	commands[i].handler(cmd, args);
     } else {
-	screen_putf("%s: Unknown command.\n", cmd);
+	screen_putf(_("%s: Unknown command.\n"), cmd);
     }
 }
 
@@ -278,17 +283,17 @@ static void
 cmd_nick(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current nick: %s\n", my_nick);
+    	screen_putf(_("Current nick: %s\n"), my_nick);
 	return;
     }
     if (strchr(args, '$') != NULL
     	|| strchr(args, '|') != NULL
 	|| strchr(args, ' ') != NULL) {
-    	screen_putf("Nick may not contain '$', '|' or space.\n");
+    	screen_putf(_("Nick may not contain '$', '|' or space.\n"));
 	return;
     }
     if (strlen(args) > 35) {
-    	screen_putf("Nick is too long (max is 35 characters)\n");
+    	screen_putf(_("Nick is too long (max is 35 characters)\n"));
 	return;
     }
     free(my_nick);
@@ -299,15 +304,15 @@ static void
 cmd_description(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current description: %s\n", my_description);
+    	screen_putf(_("Current description: %s\n"), my_description);
 	return;
     }
     if (strchr(args, '$') != NULL || strchr(args, '|') != NULL) {
-    	screen_putf("Description may not contain '$' or '|' characters\n");
+    	screen_putf(_("Description may not contain '$' or '|' characters\n"));
 	return;
     }
     if (strlen(args) > 35) {
-    	screen_putf("Description is too long (max is 35 characters)\n");
+    	screen_putf(_("Description is too long (max is 35 characters)\n"));
 	return;
     }
     free(my_description);
@@ -318,15 +323,15 @@ static void
 cmd_email(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current e-mail: %s\n", my_email);
+    	screen_putf(_("Current e-mail: %s\n"), my_email);
 	return;
     }
     if (strchr(args, '$') != NULL || strchr(args, '|') != NULL) {
-    	screen_putf("E-mail may not contain '$' or '|' characters");
+    	screen_putf(_("E-mail may not contain '$' or '|' characters\n"));
 	return;
     }
     if (strlen(args) > 35) {
-    	screen_putf("E-mail is too long (max is 35 characters)\n");
+    	screen_putf(_("E-mail is too long (max is 35 characters)\n"));
 	return;
     }
     free(my_email);
@@ -342,19 +347,19 @@ cmd_sharedir(char *cmd, char *args)
     new_dir = get_argument(args, 0, 1);
     if (args == NULL) {
     	if (share_dir == NULL) {
-    	    screen_putf("No share directory set\n");
+    	    screen_putf(_("No share directory set\n"));
 	} else {
-    	    screen_putf("Current share directory: %s\n", share_dir);
-    	    screen_putf("Currently sharing %" PRId64 " bytes (%" PRId64 " MB).\n", my_share_size, my_share_size/(1024*1024));
+    	    screen_putf(_("Current share directory: %s\n"), share_dir);
+    	    screen_putf(_("Currently sharing %" PRId64 " bytes (%" PRId64 " MB).\n"), my_share_size, my_share_size/(1024*1024));
 	}
 	return;
     }
     if (stat(new_dir, &st) < 0) {
-    	screen_putf("%s: Cannot get status - %s\n", new_dir, errstr);
+    	screen_putf(_("%s: Cannot get file status - %s\n"), new_dir, errstr);
 	return;
     }
     if (!S_ISDIR(st.st_mode)) {
-    	screen_putf("%s: Not a directory\n", new_dir);
+    	screen_putf(_("%s: Not a directory\n"), new_dir);
 	return;
     }
     set_share_dir(new_dir); /* Ignore errors */
@@ -369,15 +374,15 @@ cmd_downloaddir(char *cmd, char *args)
 
     new_dir = get_argument(args, 0, 1);
     if (new_dir == NULL) {
-    	screen_putf("Current download directory: %s\n", download_dir);
+    	screen_putf(_("Current download directory: %s\n"), download_dir);
 	return;
     }
     if (stat(new_dir, &st) < 0) {
-    	screen_putf("%s: Cannot get status - %s\n", new_dir, errstr);
+    	screen_putf(_("%s: Cannot get file status - %s\n"), new_dir, errstr);
 	return;
     }
     if (!S_ISDIR(st.st_mode)) {
-    	screen_putf("%s: Not a directory\n", new_dir);
+    	screen_putf(_("%s: Not a directory\n"), new_dir);
 	return;
     }
     free(download_dir);
@@ -392,15 +397,15 @@ cmd_listingdir(char *cmd, char *args)
 
     new_dir = get_argument(args, 0, 1);
     if (new_dir == NULL) {
-    	screen_putf("Current listing directory: %s\n", listing_dir);
+    	screen_putf(_("Current listing directory: %s\n"), listing_dir);
 	return;
     }
     if (stat(new_dir, &st) < 0) {
-    	screen_putf("%s: Cannot get status - %s\n", new_dir, errstr);
+    	screen_putf(_("%s: Cannot get file status - %s\n"), new_dir, errstr);
 	return;
     }
     if (!S_ISDIR(st.st_mode)) {
-    	screen_putf("%s: Not a directory\n", new_dir);
+    	screen_putf(_("%s: Not a directory\n"), new_dir);
 	return;
     }
     free(listing_dir);
@@ -413,7 +418,7 @@ cmd_active(char *cmd, char *args)
     bool state;
 
     if (args == NULL) {
-    	screen_putf("Current active setting: %d\n", is_active ? 1 : 0);
+    	screen_putf(_("Current active setting: %d\n"), is_active ? 1 : 0);
 	return;
     }
     if (strcmp(args, "0") == 0) {
@@ -421,22 +426,22 @@ cmd_active(char *cmd, char *args)
     } else if (strcmp(args, "1") == 0) {
     	state = true;
     } else {
-    	screen_putf("Specify active as 0 or 1.\n");
+    	screen_putf(_("Specify active as 0 or 1.\n"));
 	return;
     }
     if (!set_active(state, listen_port))
-    	screen_putf("Active setting not changed.\n");
+    	screen_putf(_("Active setting not changed.\n"));
 }
 
 static void
 cmd_slots(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current number of slots: %d\n", my_ul_slots);
+    	screen_putf(_("Current number of slots: %d\n"), my_ul_slots);
 	return;
     }
     if (!parse_uint32(args, &my_ul_slots)) {
-    	screen_putf("Invalid slot number %s\n", args);
+    	screen_putf(_("Invalid slot number `%s'\n"), args);
 	return;
     }
 }
@@ -447,11 +452,11 @@ cmd_say(char *cmd, char *args)
     char *msg;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s MESSAGE..\n", cmd);
+    	screen_putf(_("Usage: %s MESSAGE..\n"), cmd);
 	return;
     }
     if (hub_state < DC_HUB_LOGGED_IN) {
-    	screen_putf("Not connected.\n");
+    	screen_putf(_("Not connected.\n"));
 	return;
     }
     msg = escape_message(args);
@@ -479,17 +484,17 @@ cmd_msg(char *cmd, char *args)
 	}
     }
     if (args == NULL) {
-    	screen_putf("Usage: %s USER MESSAGE..\n", cmd);
+    	screen_putf(_("Usage: %s USER MESSAGE..\n"), cmd);
 	return;
     }
     if (hub_state < DC_HUB_LOGGED_IN) {
-    	screen_putf("Not connected.\n");
+    	screen_putf(_("Not connected.\n"));
 	return;
     }
 
     ui = hmap_get(hub_users, user);
     if (ui == NULL) {
-    	screen_putf("%s: No such user on this hub\n", user);
+    	screen_putf(_("%s: No such user on this hub\n"), user);
 	return;
     }
 
@@ -502,11 +507,11 @@ static void
 cmd_raw(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Usage: %s DATA..\n", cmd);
+    	screen_putf(_("Usage: %s DATA...\n"), cmd);
 	return;
     }
     if (hub_state < DC_HUB_LOCK) {
-    	screen_putf("Not connected.\n");
+    	screen_putf(_("Not connected.\n"));
 	return;
     }
     hub_putf("%s", args);
@@ -519,11 +524,11 @@ cmd_connect(char *cmd, char *args)
     struct sockaddr_in addr;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s HOST[:PORT]\n", cmd);
+    	screen_putf(_("Usage: %s HOST[:PORT]\n"), cmd);
 	return;
     }
     if (hub_state != DC_HUB_DISCONNECTED) {
-    	screen_putf("Already connected or connecting, disconnect first.\n");
+    	screen_putf(_("Already connected or connecting, disconnect first.\n"));
 	return;
     }
 
@@ -531,7 +536,7 @@ cmd_connect(char *cmd, char *args)
     if (portstr != NULL) {
     	*portstr = '\0';
     	if (!parse_uint16(portstr+1, &addr.sin_port)) {
-	    screen_putf("Invalid port number %s\n", portstr);
+	    screen_putf(_("Invalid port number %s\n"), portstr);
 	    return;
 	}
 	addr.sin_port = htons(addr.sin_port);
@@ -542,10 +547,10 @@ cmd_connect(char *cmd, char *args)
     if (!inet_aton(args, &addr.sin_addr)) {
     	struct hostent *he;
 
-	screen_putf("Looking up IP address for %s\n", args);
+	screen_putf(_("Looking up IP address for %s\n"), args);
 	he = gethostbyname(args);
 	if (he == NULL) {
-    	    screen_putf("%s: Cannot look up - %s\n", args, hstrerror(h_errno));
+    	    screen_putf(_("%s: Cannot look up address - %s\n"), args, hstrerror(h_errno));
 	    return;
 	}
     	addr.sin_addr = *(struct in_addr *) he->h_addr;
@@ -560,9 +565,9 @@ static void
 cmd_disconnect(char *cmd, char *args)
 {
     if (hub_state == DC_HUB_DISCONNECTED) {
-    	warn("Not connected.\n");
+    	warn(_("Not connected.\n"));
     } else {
-    	warn("Disconnecting from hub.\n");
+    	warn(_("Disconnecting from hub.\n"));
 	hub_disconnect();
     }
 }
@@ -573,16 +578,16 @@ cmd_grantslot(char *cmd, char *args)
     DCUserInfo *ui;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s USER\n", cmd);
+    	screen_putf(_("Usage: %s USER\n"), cmd);
 	return;
     }
     if (hub_state != DC_HUB_LOGGED_IN) {
-    	screen_putf("Not connected.\n");
+    	screen_putf(_("Not connected.\n"));
 	return;
     }
     ui = hmap_get(hub_users, args);
     if (ui == NULL) {
-    	screen_putf("%s: No such user on this hub\n", args);
+    	screen_putf(_("%s: No such user on this hub\n"), args);
 	return;
     }
     ui->slot_granted = true;
@@ -601,6 +606,7 @@ browse_none(void)
 	user_info_free(browse_user);
 	browse_user = NULL;
     }
+    browsing_myself = false;
 }
 
 static void
@@ -611,8 +617,8 @@ cmd_browse(char *cmd, char *args)
     struct stat st;
 
     if (args == NULL) {
-    	if (browse_user == NULL) {
-	    screen_putf("Not browsing any user.\n");
+    	if (!browsing_myself && browse_user == NULL) {
+	    screen_putf(_("Not browsing any user.\n"));
 	    return;
 	}
 	browse_none();
@@ -621,20 +627,25 @@ cmd_browse(char *cmd, char *args)
     }
 
     if (strcmp(my_nick, args) == 0) {
-    	screen_putf("Cannot browse myself.\n");
+	browse_none();
+	browse_list = our_filelist;
+	browse_path = xstrdup("/");
+	browse_user = NULL;
+	browsing_myself = true;
+	update_prompt();
 	return;
     }
 
     ui = hmap_get(hub_users, args);
     if (ui == NULL) {
-        screen_putf("%s: No such user on this hub\n", args);
+        screen_putf(_("%s: No such user on this hub\n"), args);
         return;
     }
 
     filename = xasprintf("%s/%s.MyList.DcLst", listing_dir, ui->nick);
     if (stat(filename, &st) < 0) {
 	if (errno != ENOENT) {
-            screen_putf("%s: stat failed - %s\n", filename, errstr);
+            screen_putf(_("%s: Cannot get file status - %s\n"), filename, errstr);
             free(filename);
             return;
 	}
@@ -651,10 +662,11 @@ cmd_browse(char *cmd, char *args)
     	if (!has_user_conn(ui, DC_DIR_RECEIVE) && ui->conn_count < DC_USER_MAX_CONN)
 	    hub_connect_user(ui); /* Ignore errors */
 	else
-	    screen_putf("No free connections. Queued file for download.\n");
+	    screen_putf(_("No free connections. Queued file for download.\n"));
 
 	browse_none();
 	browse_user = ui;
+	browsing_myself = false;
 	ui->refcount++;
 	update_prompt();
 	return;
@@ -664,6 +676,7 @@ cmd_browse(char *cmd, char *args)
     browse_list = filelist_open(filename);
     browse_path = xstrdup("/");
     browse_user = ui;
+    browsing_myself = false;
     ui->refcount++;
     free(filename);
     update_prompt();
@@ -674,13 +687,14 @@ cmd_pwd(char *cmd, char *args)
 {
     if (browse_list == NULL) {
 	if (browse_user == NULL) {
-	    screen_putf("Not browsing any user.\n");
+	    screen_putf(_("Not browsing any user.\n"));
 	} else {
-	    screen_putf("(%s) Waiting for file list.\n", browse_user->nick);
+	    screen_putf(_("(%s) Waiting for file list.\n"), browse_user->nick);
 	}
-	return;
     } else {
-	screen_putf("(%s) %s\n", browse_user->nick, browse_path);
+	screen_putf(_("(%s) %s\n"),
+		browsing_myself ? my_nick : browse_user->nick,
+		browse_path);
     }
 }
 
@@ -692,7 +706,7 @@ cmd_cd(char *cmd, char *args)
     char *path;
 
     if (browse_list == NULL) {
-	screen_putf("Not browsing any user.\n");
+	screen_putf(_("Not browsing any user.\n"));
 	return;
     }
 
@@ -706,12 +720,12 @@ cmd_cd(char *cmd, char *args)
 
     node = filelist_lookup(browse_list, path);
     if (node == NULL) {
-    	screen_putf("%s: No such file or directory\n", args);
+    	screen_putf(_("%s: No such file or directory\n"), args);
 	free(path);
 	return;
     }
     if (node->type != DC_TYPE_DIR) {
-    	screen_putf("%s: Not a directory\n", args);
+    	screen_putf(_("%s: Not a directory\n"), args);
 	free(path);
 	return;
     }
@@ -728,7 +742,7 @@ cmd_find(char *cmd, char *args)
     DCFileList *node;
 
     if (browse_list == NULL) {
-	screen_putf("Not browsing any user.\n");
+	screen_putf(_("Not browsing any user.\n"));
 	return;
     }
     arg = get_argument(args, 0, 1);
@@ -741,7 +755,7 @@ cmd_find(char *cmd, char *args)
 
     node = filelist_lookup(browse_list, path);
     if (node == NULL) {
-	screen_putf("%s: No such file or directory\n", path);
+	screen_putf(_("%s: No such file or directory\n"), path);
 	free(path);
 	return;
     }
@@ -760,7 +774,7 @@ cmd_ls(char *cmd, char *args)
     /* XXX: this code can probably be cleaned up a little... */
 
     if (browse_list == NULL) {
-	screen_putf("Not browsing any user.\n");
+	screen_putf(_("Not browsing any user.\n"));
 	return;
     }
     arg = get_argument(args, 0, 1);
@@ -788,7 +802,7 @@ cmd_ls(char *cmd, char *args)
 
     node = filelist_lookup(browse_list, path);
     if (node == NULL) {
-	screen_putf("%s: No such file or directory\n", path);
+	screen_putf(_("%s: No such file or directory\n"), path);
 	free(path);
 	return;
     }
@@ -804,7 +818,7 @@ append_download_dir(DCUserInfo *ui, DCFileList *node, char *base_path, uint64_t
     	char *path = filelist_get_path(node);
 
 	if (ptrv_find(ui->download_queue, path, (comparison_fn_t) queued_file_cmp) >= 0) {
-	    screen_putf("Queue already contains this file, ignoring\n");
+	    screen_putf(_("Queue already contains this file, ignoring\n"));
 	    free(path);
 	    return false;
 	}
@@ -848,11 +862,15 @@ cmd_get(char *cmd, char *args)
 
     file_name = get_argument(args, 0, 1);
     if (file_name == NULL) {
-    	screen_putf("Usage: %s FILE\n", cmd);
+    	screen_putf(_("Usage: %s FILE\n"), cmd);
 	return;
     }
     if (browse_list == NULL) {
-	screen_putf("Not browsing any user.\n");
+	screen_putf(_("Not browsing any user.\n"));
+	return;
+    }
+    if (browsing_myself) {
+	screen_putf(_("Cannot download files from myself.\n"));
 	return;
     }
 
@@ -860,26 +878,35 @@ cmd_get(char *cmd, char *args)
     free(file_name);
     node = filelist_lookup(browse_list, path);
     if (node == NULL) {
-	screen_putf("%s: No such file or directory\n", path);
+	screen_putf(_("%s: No such file or directory\n"), path);
 	free(path);
 	return;
     }
     free(path);
 
     if (append_download_dir(browse_user, node, browse_path, &bytes, &files)) {
+	char *fmt;
+
+	fmt = ngettext("Downloading %s (%" PRId64 " bytes in %d file)\n",
+		"Downloading %s (%" PRId64 " bytes in %d files)\n", files);
     	path = filelist_get_path(node);
-	screen_putf("Downloading %s (%" PRId64 " bytes in %d %s)\n", path, bytes, files, files > 1 ? "files" : "file");
+	screen_putf(fmt, path, bytes, files);
 	free(path);
     } else {
     	if (browse_user->download_queue->cur == 0) {
-	    screen_putf("No files to download (empty directories).");
+	    screen_putf(_("No files to download (empty directories).\n"));
 	    return;
 	}
     }
     if (!has_user_conn(browse_user, DC_DIR_RECEIVE) && browse_user->conn_count < DC_USER_MAX_CONN)
     	hub_connect_user(browse_user); /* Ignore errors */
-    else
-	screen_putf("No free connections. Queued file(s) for download.\n");
+    else {
+	char *msg;
+
+	msg = ngettext("No free connections. Queued file for download.\n",
+		"No free connections. Queued files for download.\n", files);
+	screen_putf(msg);
+    }
 }
 
 static void
@@ -889,17 +916,17 @@ cmd_queue(char *cmd, char *args)
     DCUserInfo *ui;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s USER\n", cmd);
+    	screen_putf(_("Usage: %s USER\n"), cmd);
 	return;
     }
     ui = hmap_get(hub_users, args);
     if (ui == NULL) {
-        screen_putf("%s: No such user on this hub\n", args);
+        screen_putf(_("%s: No such user on this hub\n"), args);
 	return;
     }
     for (c = 0; c < ui->download_queue->cur; c++) {
 	DCQueuedFile *queued = ui->download_queue->buf[c];
-	screen_putf("%d. [ %s ] %s\n",
+	screen_putf(_("%d. [ %s ] %s\n"),
 		c+1,
 		queued->base_path,
 		queued->filename + strlen(queued->base_path));
@@ -914,7 +941,7 @@ cmd_unqueue(char *cmd, char *args)
     int32_t sp, ep;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s USER RANGE\n", cmd);
+    	screen_putf(_("Usage: %s USER RANGE\n"), cmd);
 	return;
     }
 
@@ -928,7 +955,7 @@ cmd_unqueue(char *cmd, char *args)
         t2 = strchr(t, '-');
         if (t2 == NULL) { /* No dash. */
             if (!parse_uint32(t, &sp)) {
-                screen_putf("Invalid range1\n");
+                screen_putf(_("Invalid range\n"));
                 return;
             }
 	    ep = sp+1;
@@ -936,13 +963,13 @@ cmd_unqueue(char *cmd, char *args)
             sp = 1;
             t++;
             if (!parse_uint32(t, &ep)) {
-                screen_putf("Invalid range2\n");
+                screen_putf(_("Invalid range\n"));
                 return;
             }
         } else if (t2[1] == '\0') { /* Ends with dash. */
             *t2 = '\0';
             if (!parse_uint32(t, &sp)) {
-                screen_putf("Invalid range3\n");
+                screen_putf(_("Invalid range\n"));
                 return;
             }
 	    ep = -1;
@@ -950,11 +977,11 @@ cmd_unqueue(char *cmd, char *args)
             *t2 = '\0';
             t2++;
             if (!parse_uint32(t, &sp)) {
-                screen_putf("Invalid range4\n");
+                screen_putf(_("Invalid range\n"));
                 return;
             }
             if (!parse_uint32(t2, &ep)) {
-                screen_putf("Invalid range5\n");
+                screen_putf(_("Invalid range\n"));
                 return;
             }
         }
@@ -962,12 +989,12 @@ cmd_unqueue(char *cmd, char *args)
 
     user = hmap_get(hub_users, args);
     if (user == NULL) {
-        screen_putf("%s: No such user on this hub\n", args);
+        screen_putf(_("%s: No such user on this hub\n"), args);
         return;
     }
 
     if (sp == 0 || ep == 0 || sp > user->download_queue->cur || ep > user->download_queue->cur) {
-        screen_putf("Invalid range\n");
+        screen_putf(_("Invalid range\n"));
         return;
     }
     sp--;
@@ -1012,14 +1039,14 @@ cmd_speed(char *cmd, char *args)
     if (args == NULL) {
     	uint32_t c;
 
-        screen_putf("Current speed: %s\n", my_speed);
-        screen_putf("Speed should be one of:\n");
+        screen_putf(_("Current speed: %s\n"), my_speed);
+        screen_putf(_("Speed should be one of\n"));
 	for (c = 0; c < SPEED_COUNT; c++)
 	    screen_putf("  %s\n", speeds[c]);
 	return;
     }
     if (strchr(args, '$') != NULL || strchr(args, '|') != NULL) {
-        screen_putf("Speed may not contain '$' or '|' characters");
+        screen_putf(_("Speed may not contain '$' or '|' characters"));
         return;
     }
     free(my_speed);
@@ -1031,16 +1058,16 @@ cmd_password(char *cmd, char *args)
 {
     if (args == NULL) {
     	if (my_password != NULL) {
-    	    screen_putf("Removing current password.\n");
+    	    screen_putf(_("Removing current password.\n"));
 	    free(my_password);
 	    my_password = NULL;
 	} else {
-	    screen_putf("No current password set - not removed.\n");
+	    screen_putf(_("No current password set - not removed.\n"));
 	}
 	return;
     }
     if (strchr(args, '|') != NULL) {
-    	screen_putf("Password may not contain '|' character");
+    	screen_putf(_("Password may not contain '|' character"));
 	return;
     }
     free(my_password);
@@ -1068,7 +1095,7 @@ cmd_who(char *cmd, char *args)
     StrBuf *out;
 
     if (hub_state < DC_HUB_LOGGED_IN) {
-    	screen_putf("Not connected.\n");
+    	screen_putf(_("Not connected.\n"));
 	return;
     }
 
@@ -1077,15 +1104,15 @@ cmd_who(char *cmd, char *args)
 
 	ui = hmap_get(hub_users, args);
 	if (ui == NULL) {
-    	    screen_putf("%s: No such user on this hub\n", args);
+    	    screen_putf(_("%s: No such user on this hub\n"), args);
 	} else {
-    	    screen_putf("Nick: %s\n", ui->nick);
-    	    screen_putf("Description: %s\n", ui->description);
-    	    screen_putf("Speed: %s\n", ui->speed);
-    	    screen_putf("Level: %d\n", ui->level);
-    	    screen_putf("E-mail: %s\n", ui->email);
-    	    screen_putf("Operator: %d\n", ui->is_operator);
-    	    screen_putf("Share Size: %" PRId64 " bytes (%" PRId64 " MB)\n", ui->share_size, ui->share_size/(1024*1024));
+    	    screen_putf(_("Nick: %s\n"), ui->nick);
+    	    screen_putf(_("Description: %s\n"), ui->description);
+    	    screen_putf(_("Speed: %s\n"), ui->speed);
+    	    screen_putf(_("Level: %d\n"), ui->level);
+    	    screen_putf(_("E-mail: %s\n"), ui->email);
+    	    screen_putf(_("Operator: %d\n"), ui->is_operator);
+    	    screen_putf(_("Share Size: %" PRId64 " bytes (%" PRId64 " MB)\n"), ui->share_size, ui->share_size/(1024*1024));
 	}
     	return;
     }
@@ -1131,11 +1158,11 @@ static void
 cmd_tag(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current tag: %s\n", my_tag);
+    	screen_putf(_("Current tag: %s\n"), my_tag);
 	return;
     }
     if (strchr(args, '$') != NULL || strchr(args, '|') != NULL) {
-    	screen_putf("Tag may not contain '$' or '|' characters");
+    	screen_putf(_("Tag may not contain '$' or '|' characters"));
 	return;
     }
     free(my_tag);
@@ -1158,8 +1185,10 @@ cmd_transfers(char *cmd, char *args)
 
     format = xasprintf("%%-%ds  %%s\n", maxlen);
 
-    if (time(&now) < 0)
-    	warn("Cannot get current time - %s\n", errstr);
+    if (time(&now) < 0) {
+    	warn(_("Cannot get current time - %s\n"), errstr);
+	return;
+    }
 
     hmap_iterator(user_conns, &it);
     while (it.has_next(&it)) {
@@ -1173,7 +1202,7 @@ cmd_transfers(char *cmd, char *args)
 	free(status);
     }
 
-    screen_putf("Upload slots: %d/%d  Download slots: %d/unlimited\n", used_ul_slots, my_ul_slots, used_dl_slots);
+    screen_putf(_("Upload slots: %d/%d  Download slots: %d/unlimited\n"), used_ul_slots, my_ul_slots, used_dl_slots);
 
     free(format);
 }
@@ -1184,13 +1213,13 @@ cmd_cancel(char *cmd, char *args)
     DCUserConn *uc;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s CONNECTION\n", cmd);
+    	screen_putf(_("Usage: %s CONNECTION\n"), cmd);
 	return;
     }
 
     uc = hmap_get(user_conns, args);
     if (uc == NULL) {
-    	screen_putf("%s: No such user connection.\n", args);
+    	screen_putf(_("%s: No such user connection.\n"), args);
 	return;
     }
 
@@ -1201,36 +1230,69 @@ static void
 cmd_debug(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Current debug level: %d\n", debug_level);
+    	screen_putf(_("Current debug level: %d\n"), debug_level);
 	return;
     }
     if (!parse_uint32(args, &debug_level)) {
-    	screen_putf("%s: Invalid integer\n", args);
+    	screen_putf(_("%s: Invalid integer\n"), args);
 	return;
     }
 }
 
+static void
+cmd_listenaddr(char *cmd, char *args)
+{
+    struct sockaddr_in addr;
+
+    if (args == NULL) {
+	if (force_listen_addr.s_addr == INADDR_NONE) {
+	    screen_putf(_("Listening address not set.\n"));
+	} else {
+	    force_listen_addr.s_addr = INADDR_NONE;
+	    screen_putf(_("Removing listening address.\n"));
+	}
+	//screen_putf(_("Current listening address: %s\n"), sockaddr_in_str(&local_addr));
+	return;
+    }
+
+    if (!inet_aton(args, &addr.sin_addr)) {
+	struct hostent *he;
+
+	screen_putf(_("Looking up IP address for %s\n"), args);
+	he = gethostbyname(args);
+	if (he == NULL) {
+	    screen_putf(_("%s: Cannot look up address - %s\n"), args, hstrerror(h_errno));
+	    return;
+	}
+
+	addr.sin_addr = *(struct in_addr *) he->h_addr;
+    }
+
+    force_listen_addr = addr.sin_addr;
+    screen_putf(_("Listening address set to %s.\n"), inet_ntoa(force_listen_addr));
+}
+
 static void
 cmd_listenport(char *cmd, char *args)
 {
     uint16_t port;
     if (args == NULL) {
-    	screen_putf("Current listen port: %d\n", listen_port);
+    	screen_putf(_("Current listening port: %d\n"), listen_port);
 	return;
     }
     if (!parse_uint16(args, &port)) {
-    	screen_putf("%s: Invalid integer\n", args);
+    	screen_putf(_("%s: Invalid integer\n"), args);
 	return;
     }
     if (!set_active(is_active, port))
-    	screen_putf("Active setting not changed.\n");
+    	screen_putf(_("Active setting not changed.\n"));
 }
 
 static void
 cmd_search(char *cmd, char *args)
 {
     if (args == NULL) {
-    	screen_putf("Usage: %s STRING..\n", cmd);
+    	screen_putf(_("Usage: %s STRING...\n"), cmd);
 	return;
     }
     add_search_request(args); /* Ignore errors */
@@ -1244,12 +1306,12 @@ cmd_results(char *cmd, char *args)
     if (args == NULL) {
     	for (c = 0; c < our_searches->cur; c++) {
     	    DCSearchRequest *sd = our_searches->buf[c];
-	    screen_putf("%d. Results: %d\n", c+1, sd->responses->cur);
+	    screen_putf(_("%d. Results: %d\n"), c+1, sd->responses->cur);
 	}
     } else {
     	DCSearchRequest *sd;
 	if (!parse_uint32(args, &c) || c == 0 || c-1 >= our_searches->cur) {
-    	    screen_putf("Invalid search index.\n");
+    	    screen_putf(_("Invalid search index.\n"));
 	    return;
 	}
 	sd = our_searches->buf[c-1];
@@ -1276,11 +1338,11 @@ cmd_unsearch(char *cmd, char *args)
     uint32_t index;
 
     if (args == NULL) {
-    	screen_putf("Usage: %s INDEX\n", cmd);
+    	screen_putf(_("Usage: %s INDEX\n"), cmd);
 	return;
     }
     if (!parse_uint32(args, &index) || index == 0 || index-1 >= our_searches->cur) {
-    	screen_putf("Invalid search index.\n");
+    	screen_putf(_("Invalid search index.\n"));
 	return;
     }
     
@@ -1292,11 +1354,13 @@ cmd_unsearch(char *cmd, char *args)
 void
 update_prompt(void)
 {
-    if (browse_user != NULL) {
+    if (browsing_myself || browse_user != NULL) {
+	char *nick = browsing_myself ? my_nick : browse_user->nick;
+	
 	if (browse_list == NULL) {
-	    set_screen_prompt("%s:(%s)> ", PACKAGE, browse_user->nick);
+	    set_screen_prompt("%s:(%s)> ", PACKAGE, nick);
 	} else {
-	    set_screen_prompt("%s:%s:%s> ", PACKAGE, browse_user->nick, browse_path);
+	    set_screen_prompt("%s:%s:%s> ", PACKAGE, nick, browse_path);
 	}
     } else {
 	set_screen_prompt("%s> ", PACKAGE);
