@@ -18,10 +18,13 @@
  */
 
 #include <config.h>
-#include <ctype.h>
-#include "common/memory.h"
-#include "microdc.h"
+#include <ctype.h>		/* C89 */
+#include "xalloc.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/strbuf.h"
+#include "microdc.h"
 
 /* Decode data from a $Lock command sent by either a client or a hub.
  * The returned value must be freed using free(). The length of the
@@ -36,7 +39,7 @@ decode_lock(const uint8_t *lock, int locklen, uint32_t basekey)
     int d;
 
     if (locklen < 3) {
-    	screen_putf("Invalid $Lock message: Key to short\n");
+    	screen_putf(_("Invalid $Lock message: Key to short\n"));
     	return xstrdup("");
     }
 
