@@ -18,27 +18,30 @@
  */
 
 #include <config.h>
-#include <assert.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <stdlib.h>
-#include <dirent.h>
-#include <sys/types.h>
-#include <inttypes.h>
-#include "microdc.h"
-#include "common/error.h"
-#include "full-read.h"
-#include "common/memory.h"
-#include "common/substrcmp.h"
-#include "minmax.h"
+#include <assert.h>		/* ? */
+#include <sys/types.h>		/* ? */
+#include <sys/stat.h>		/* ? */
+#include <unistd.h>		/* POSIX */
+#include <fcntl.h>		/* ? */
+#include <stdlib.h>		/* C89 */
+#include <dirent.h>		/* ? */
+#include <sys/types.h>		/* ? */
+#include <inttypes.h>		/* ? */
+#include "full-read.h"		/* Gnulib */
+#include "xalloc.h"		/* Gnulib */
+#include "minmax.h"		/* Gnulib */
+#include "full-write.h"		/* Gnulib */
+#include "xvasprintf.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/common.h"
+#include "common/error.h"
+#include "common/substrcmp.h"
 #include "common/intparse.h"
 #include "common/strbuf.h"
-#include "full-write.h"
 #include "common/strleftcmp.h"
-#include "xvasprintf.h"
+#include "microdc.h"
 
 #define MAX_DIR_DEPTH 32
 
@@ -377,7 +380,7 @@ filelist_open(const char *filename)
     ssize_t res;
 
     if (stat(filename, &st) < 0) {
-    	screen_putf("%s: Cannot get status - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot get file status - %s\n"), filename, errstr);
 	return NULL;
     }
     contents = malloc(st.st_size);
@@ -387,26 +390,26 @@ filelist_open(const char *filename)
     }
     fd = open(filename, O_RDONLY);
     if (fd < 0) {
-    	screen_putf("%s: Cannot open - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot open file for reading - %s\n"), filename, errstr);
 	return NULL;
     }
     res = full_read(fd, contents, st.st_size);
     if (res < st.st_size) {
     	if (res < 0)
-    	    screen_putf("%s: Cannot read - %s\n", filename, errstr);
+    	    screen_putf(_("%s: Cannot read from file - %s\n"), filename, errstr);
 	else
-	    screen_putf("%s: Premature end when reading file\n", filename);
+	    screen_putf(_("%s: Premature end of file\n"), filename);	/* XXX: really: file was truncated? */
 	free(contents);
 	if (close(fd) < 0)
-    	    screen_putf("%s: Cannot close - %s\n", filename, errstr);
+    	    screen_putf(_("%s: Cannot close file - %s\n"), filename, errstr);
 	return NULL;
     }
     if (close(fd) < 0)
-    	screen_putf("%s: Cannot close - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot close file - %s\n"), filename, errstr);
     decoded = huffman_decode(contents, st.st_size, &decoded_len);
     free(contents);
     if (decoded == NULL) {
-    	screen_putf("%s: Cannot decode - Invalid data\n", filename);
+    	screen_putf(_("%s: Invalid data, cannot decode\n"), filename);
 	return NULL;
     }
     root = parse_decoded_dclst(decoded, decoded_len);
@@ -422,7 +425,7 @@ dir_to_filelist(DCFileList *parent, const char *path)
 
     dp = opendir(path);
     if (dp == NULL) {
-    	screen_putf("%s: Cannot open directory - %s\n", path, errstr);
+    	screen_putf(_("%s: Cannot open directory - %s\n"), path, errstr);
 	return;
     }
     errno = 0;
@@ -437,9 +440,7 @@ dir_to_filelist(DCFileList *parent, const char *path)
 
     	fullname = catfiles(path, ep->d_name);
     	if (stat(fullname, &st) < 0) {
-	    screen_putf("%s: Cannot get status - %s\n", fullname, errstr);
-	    if (closedir(dp) < 0)
-    		screen_putf("%s: Cannot close directory - %s\n", path, errstr);
+	    screen_putf(_("%s: Cannot get file status - %s\n"), fullname, errstr);
 	    free(fullname);
 	    continue;
 	}
@@ -458,14 +459,14 @@ dir_to_filelist(DCFileList *parent, const char *path)
 	    hmap_put(parent->u.dir.children, node->name, node);
 	}
 	else {
-	    screen_putf("%s: Not a regular file or directory, ignoring\n", fullname);
+	    screen_putf(_("%s: Not a regular file or directory, ignoring\n"), fullname);
 	}
 	free(fullname);
     }
     if (errno != 0)
-    	screen_putf("%s: Cannot read directory - %s\n", path, errstr);
+    	screen_putf(_("%s: Cannot read directory - %s\n"), path, errstr);
     if (closedir(dp) < 0)
-    	screen_putf("%s: Cannot close directory - %s\n", path, errstr);
+    	screen_putf(_("%s: Cannot close directory - %s\n"), path, errstr);
 }
 
 static void
@@ -506,7 +507,7 @@ filelist_create(const char *basedir)
 
     sb = strbuf_new();
     if (basedir != NULL) {
-    	screen_putf("Scanning directory %s\n", basedir);
+    	screen_putf(_("Scanning directory %s\n"), basedir);
     	dir_to_filelist(root, basedir);
 	filelist_to_string(root, sb, 0);
     }
@@ -520,10 +521,10 @@ filelist_create(const char *basedir)
     mkdirs_for_file(filename, true); /* Ignore errors */
     if (stat(filename, &st) < 0) {
     	if (errno != ENOENT)
-	    warn("%s: Cannot get status - %s\n", filename, errstr);
+	    warn(_("%s: Cannot get file status - %s\n"), filename, errstr);
     } else {
 	if (unlink(filename) < 0)
-	    warn("%s: Cannot remove - %s\n", filename, errstr);
+	    warn(_("%s: Cannot remove file - %s\n"), filename, errstr);
     }
     i = ptrv_find(delete_files, filename, (comparison_fn_t) strcmp);
     if (i >= 0)
@@ -531,7 +532,7 @@ filelist_create(const char *basedir)
 
     fd = open(filename, O_CREAT|O_EXCL|O_WRONLY, 0666);
     if (fd < 0) {
-    	screen_putf("%s: Cannot open for writing - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot open file for writing - %s\n"), filename, errstr);
 	filelist_free(root);
 	free(outdata);
 	free(filename);
@@ -541,14 +542,14 @@ filelist_create(const char *basedir)
     	ptrv_append(delete_files, xstrdup(filename));
 
     if (full_write(fd, outdata, len) < len) {
-    	screen_putf("%s: Cannot write - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot write to file - %s\n"), filename, errstr);
 	filelist_free(root); 
 	free(outdata);
 	free(filename);
 	return false;
     }
     if (close(fd) < 0)
-    	screen_putf("%s: Cannot close - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot close file - %s\n"), filename, errstr);
     free(filename);
     free(outdata);
 
