@@ -29,6 +29,9 @@
 #include "xvasprintf.h"		/* Gnulib */
 #include "xstrndup.h"		/* Gnulib */
 #include "xalloc.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/error.h"
 #include "common/intparse.h"
 #include "common/strleftcmp.h"
@@ -70,6 +73,27 @@ say_user_completion_generator(const char *base, int state, DCCompletionEntry *ce
     return false;
 }
 
+char *
+user_or_myself_completion_generator(const char *base, int state)
+{
+    static HMapIterator it;
+
+    if (state == 0) {
+    	hmap_iterator(hub_users, &it);
+	if (strleftcmp(base, my_nick) == 0)
+	    return xstrdup(my_nick);
+    }
+
+    while (it.has_next(&it)) {
+    	DCUserInfo *ui = it.next(&it);
+	if (strleftcmp(base, ui->nick) == 0)
+	    return xstrdup(ui->nick);
+    }
+
+
+    return NULL;
+}
+
 char *
 user_completion_generator(const char *base, int state)
 {
@@ -110,12 +134,12 @@ user_info_new(const char *nick)
     info->conn_count = 0;
 
     /* XXX Find existing connections to this user... */
-    ucname = xasprintf("%s|%s", nick, "UL");
+    ucname = xasprintf(_("%s|UL"), nick);
     info->conn[info->conn_count] = hmap_get(user_conns, ucname);
     if (info->conn[info->conn_count] != NULL)
     	info->conn_count++;
     free(ucname);
-    ucname = xasprintf("%s|%s", nick, "DL");
+    ucname = xasprintf(_("%s|DL"), nick);
     info->conn[info->conn_count] = hmap_get(user_conns, ucname);
     if (info->conn[info->conn_count] != NULL)
     	info->conn_count++;
@@ -162,7 +186,7 @@ hub_putf(const char *format, ...)
     va_end(args);
 
     if (res < 0) {
-    	warn("Cannot append to hub sendq - %s\n", errstr);
+    	warn(_("Cannot append to hub send queue - %s\n"), errstr);
 	hub_disconnect();
 	return false;
     }
@@ -171,7 +195,7 @@ hub_putf(const char *format, ...)
 
     res = byteq_write(hub_sendq, hub_socket);
     if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	warn_socket_eof(res, "send to", "hub");
+    	warn_socket_error(res, true, _("hub"));
 	hub_disconnect();
 	return false;
     }
@@ -187,22 +211,22 @@ hub_connect(struct sockaddr_in *addr)
 {
     hub_socket = socket(PF_INET, SOCK_STREAM, 0);
     if (hub_socket < 0) {
-        warn("Cannot create socket - %s\n", errstr);
+        warn(_("Cannot create socket - %s\n"), errstr);
 	hub_disconnect();
         return;
     }
 
     /* Set non-blocking I/O on socket. */
     if (fd_set_nonblock_flag(hub_socket, true) < 0) {
-        warn("Cannot set socket flags - %s\n", errstr);
+        warn(_("Cannot set socket flags - %s\n"), errstr);
 	hub_disconnect();
 	return;
     }
 
-    screen_putf("Connecting to hub on %s.\n", sockaddr_in_str(addr));
+    screen_putf(_("Connecting to hub on %s.\n"), sockaddr_in_str(addr));
     if (connect(hub_socket, (struct sockaddr *) addr, sizeof(struct sockaddr_in)) < 0
     	    && errno != EINPROGRESS) {
-        warn("Cannot connect - %s\n", errstr);
+        warn(_("Cannot connect - %s\n"), errstr);
 	hub_disconnect();
         return;
     }
@@ -217,11 +241,11 @@ hub_disconnect(void)
 {
     if (hub_socket >= 0) {
     	if (hub_state > DC_HUB_CONNECT)
-    	    screen_putf("Shutting down hub connection.\n");
+    	    screen_putf(_("Shutting down hub connection.\n"));
     	FD_CLR(hub_socket, &read_fds);
         FD_CLR(hub_socket, &write_fds);
     	if (close(hub_socket) < 0)
-	    warn("Cannot close - %s\n", errstr);
+	    warn(_("Cannot close socket - %s\n"), errstr);
     	hub_socket = -1;
     }
     if (hub_users != NULL) {
@@ -243,38 +267,13 @@ static bool
 check_state(char *buf, DCUserState state)
 {
     if (hub_state != state) {
-	warn("Received %s message in wrong state.\n", strtok(buf, " "));
+	warn(_("Received %s message in wrong state.\n"), strtok(buf, " "));
 	hub_disconnect();
 	return false;
     }
     return true;
 }
 
-static bool
-parse_ip_and_port(char *source, struct sockaddr_in *addr, uint16_t defport) /* XXX: move util.c */
-{
-    char *port;
-
-    port = strchr(source, ':');
-    if (port == NULL) {
-    	if (defport == 0)
-	    return false;
-	addr->sin_port = htons(defport);
-    } else {
-	*port = '\0';
-	port++;
-	if (!parse_uint16(port, &addr->sin_port))
-    	    return false;
-        addr->sin_port = htons(addr->sin_port);
-    }
-
-    if (!inet_aton(source, &addr->sin_addr))
-    	return false;
-
-    addr->sin_family = AF_INET;
-    return true;
-}
-
 static void
 hub_handle_command(char *buf, uint32_t len)
 {
@@ -286,7 +285,7 @@ hub_handle_command(char *buf, uint32_t len)
 
         key = memmem(buf+6, len-6, " Pk=", 4);
         if (key == NULL) {
-            screen_putf("Invalid $Lock message: Missing Pk value");
+            screen_putf(_("Invalid $Lock message: Missing Pk value\n"));
 	    key = buf+len;
 	}
         key = decode_lock(buf+6, key-buf-6, DC_CLIENT_BASE_KEY);
@@ -301,25 +300,25 @@ hub_handle_command(char *buf, uint32_t len)
     }
     else if (strcmp(buf, "$GetPass") == 0) {
     	if (my_password == NULL) {
-    	    screen_putf("Hub requires password.\n");
+    	    screen_putf(_("Hub requires password.\n"));
 	    hub_disconnect();
 	    return;
 	}
-	screen_putf("Sending password to hub.\n");
+	screen_putf(_("Sending password to hub.\n"));
 	if (!hub_putf("$MyPass %s|", my_password))
 	    return;
     }
     else if (strcmp(buf, "$BadPass") == 0) {
-    	screen_putf("Password not accepted.\n");
+    	screen_putf(_("Password not accepted.\n"));
 	hub_disconnect();
     }
     else if (strcmp(buf, "$LogedIn") == 0) {
-    	screen_putf("You have received operator status.\n");
+    	screen_putf(_("You have received operator status.\n"));
     }
     else if (len >= 9 && strncmp(buf, "$HubName ", 9) == 0) {
 	free(hub_name);
 	hub_name = xstrdup(buf+9);
-        screen_putf("Hub name is %s.\n", hub_name);
+        screen_putf(_("Hub name is %s.\n"), hub_name);
     }
     else if (strcmp(buf, "$ValidateDenide") == 0) {
     	if (!check_state(buf, DC_HUB_HELLO))
@@ -328,7 +327,7 @@ hub_handle_command(char *buf, uint32_t len)
          * But shouldn't we give the client a chance to change the nick?
          * Also what happens if we receive this when correctly logged in?
          */
-    	screen_putf("Hub did not accept nick. Nick may be in use.\n");
+    	screen_putf(_("Hub did not accept nick. Nick may be in use.\n"));
     	hub_disconnect();
     }
     else if (len >= 7 && strncmp(buf, "$Hello ", 7) == 0) {
@@ -336,12 +335,12 @@ hub_handle_command(char *buf, uint32_t len)
 
     	if (hub_state == DC_HUB_HELLO) {
 	    if (strcmp(buf+7, my_nick) == 0) {
-    	    	screen_putf("Nick accepted. You are now logged in.\n");
+    	    	screen_putf(_("Nick accepted. You are now logged in.\n"));
 	    } else {
 	    	/* This probably won't happen, but better safe... */
 	    	free(my_nick);
 		my_nick = xstrdup(buf+7);
-    	    	screen_putf("Nick accepted but modified to %s. You are now logged in.\n", my_nick);
+    	    	screen_putf(_("Nick accepted but modified to %s. You are now logged in.\n"), my_nick);
 	    }
 
     	    ui = user_info_new(my_nick);
@@ -357,7 +356,7 @@ hub_handle_command(char *buf, uint32_t len)
 	    	return;
     	    hub_state = DC_HUB_LOGGED_IN;
 	} else {
-	    screen_putf("User %s logged in.\n", buf+7);
+	    screen_putf(_("User %s logged in.\n"), buf+7);
 	    ui = user_info_new(buf+7);
             hmap_put(hub_users, ui->nick, ui);
 	    if (!hub_putf("$GetINFO %s|", buf+7))
@@ -372,24 +371,24 @@ hub_handle_command(char *buf, uint32_t len)
     	buf += 8;
     	token = strsep(&buf, " ");
 	if (strcmp(token, "$ALL") != 0) {
-	    screen_putf("Invalid $MyINFO message: Missing $ALL parameter, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing $ALL parameter, ignoring\n"));
 	    return;
 	}
 
     	token = strsep(&buf, " ");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing nick parameter, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing nick parameter, ignoring\n"));
 	    return;
 	}
     	ui = hmap_get(hub_users, token);
 	if (ui == NULL) {
-	    screen_putf("Invalid $MyINFO message: Unknown user %s, ignoring\n", token);
+	    screen_putf(_("Invalid $MyINFO message: Unknown user %s, ignoring\n"), token);
 	    return;
 	}
 
     	token = strsep(&buf, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing description parameter, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing description parameter, ignoring\n"));
 	    return;
 	}
 	free(ui->description);
@@ -397,13 +396,13 @@ hub_handle_command(char *buf, uint32_t len)
 
     	token = strsep(&buf, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing description separator, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing description separator, ignoring\n"));
 	    return;
 	}
 
     	token = strsep(&buf, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing connection speed, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing connection speed, ignoring\n"));
 	    return;
 	}
 	len = strlen(token);
@@ -418,7 +417,7 @@ hub_handle_command(char *buf, uint32_t len)
 
     	token = strsep(&buf, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing e-mail address, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing e-mail address, ignoring\n"));
 	    return;
 	}
 	free(ui->email);
@@ -426,11 +425,11 @@ hub_handle_command(char *buf, uint32_t len)
 
     	token = strsep(&buf, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $MyINFO message: Missing share size, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Missing share size, ignoring\n"));
 	    return;
 	}
 	if (!parse_uint64(token, &ui->share_size)) {
-	    screen_putf("Invalid $MyINFO message: Invalid share size, ignoring\n");
+	    screen_putf(_("Invalid $MyINFO message: Invalid share size, ignoring\n"));
 	    return;
 	}
 
@@ -440,39 +439,39 @@ hub_handle_command(char *buf, uint32_t len)
 	/* XXX: Now that user's active_state may have changed, try queue again? */
     }
     else if (strcmp(buf, "$HubIsFull") == 0) {
-        screen_putf("Hub is full.\n");
+        screen_putf(_("Hub is full.\n"));
         /* DC++ does not disconnect immediately. So I guess we won't either. */
 	/* Maybe we should be expecting an "hub follow" message? */
 	/* hub_disconnect(); */
     }
     else if (len >= 3 && buf[0] == '<') {
-        screen_putf("Public: %s\n", buf);
+        screen_putf(_("Public: %s\n"), buf);
     }
     else if (len >= 5 && strncmp(buf, "$To: ", 5) == 0) {
     	char *msg;
 	msg = strchr(buf+5, '$');
 	if (msg == NULL) {
-	    screen_putf("Invalid $To message: Missing text separator, ignoring\n");
+	    screen_putf(_("Invalid $To message: Missing text separator, ignoring\n"));
 	    return;
         }
         *msg = '\0';
 	msg++;
-        screen_putf("Private: [%s] %s\n", buf+5, msg);
+        screen_putf(_("Private: [%s] %s\n"), buf+5, msg);
     }
     else if (len >= 13 && strncmp(buf, "$ConnectToMe ", 13) == 0) {
 	struct sockaddr_in addr;
 
     	buf += 13;
 	if (strsep(&buf, " ") == NULL) {
-	    screen_putf("Invalid $ConnectToMe message: Missing or invalid nick\n");
+	    screen_putf(_("Invalid $ConnectToMe message: Missing or invalid nick\n"));
 	    return;
     	}
     	if (!parse_ip_and_port(buf, &addr, 0)) {
-	    screen_putf("Invalid $ConnectToMe message: Invalid address specification.\n");
+	    screen_putf(_("Invalid $ConnectToMe message: Invalid address specification.\n"));
 	    return;
         }
 
-	screen_putf("Connecting to user on %s\n", sockaddr_in_str(&addr));
+	screen_putf(_("Connecting to user on %s\n"), sockaddr_in_str(&addr));
     	user_connection_new(&addr, -1);
     }
     else if (len >= 16 && strncmp(buf, "$RevConnectToMe ", 16) == 0) {
@@ -481,27 +480,27 @@ hub_handle_command(char *buf, uint32_t len)
 
     	nick = strtok(buf+16, " ");
 	if (nick == NULL) {
-	    screen_putf("Invalid $RevConnectToMe message: Missing nick parameter\n");
+	    screen_putf(_("Invalid $RevConnectToMe message: Missing nick parameter\n"));
 	    return;
         }
     	if (strcmp(nick, my_nick) == 0) {
-	    screen_putf("Invalid $RevConnectToMe message: Remote nick is our nick\n");
+	    screen_putf(_("Invalid $RevConnectToMe message: Remote nick is our nick\n"));
 	    return;
 	}
         ui = hmap_get(hub_users, nick);
         if (ui == NULL) {
-	    screen_putf("Invalid $RevConnectToMe message: Unknown user %s, ignoring\n", nick);
+	    screen_putf(_("Invalid $RevConnectToMe message: Unknown user %s, ignoring\n"), nick);
 	    return;
         }
 
 	if (ui->conn_count >= DC_USER_MAX_CONN) {
-	    screen_putf("No more connections to user %s allowed.\n", ui->nick);
+	    screen_putf(_("No more connections to user %s allowed.\n"), ui->nick);
 	    return;
 	}
 
     	if (!is_active) {
     	    if (ui->active_state == DC_ACTIVE_SENT_PASSIVE) {
-    		screen_putf("User %s is also passive. Cannot establish connection.\n", ui->nick);
+    		screen_putf(_("User %s is also passive. Cannot establish connection.\n"), ui->nick);
 		/* We could set this to DC_ACTIVE_UNKNOWN too. This would mean
 		 * we would keep sending RevConnectToMe next time the download
 		 * queue is modified or some timer expired and told us to retry
@@ -573,7 +572,7 @@ hub_handle_command(char *buf, uint32_t len)
     	screen_putf("User %s quits.\n", buf+6);
 	ui = hmap_remove(hub_users, buf+6);
 	if (ui == NULL) {
-	    screen_putf("Invalid $Quit message: Unknown user %s.\n", buf+6);
+	    screen_putf(_("Invalid $Quit message: Unknown user %s.\n"), buf+6);
 	    return;
 	}
 	user_info_free(ui);
@@ -585,17 +584,17 @@ hub_handle_command(char *buf, uint32_t len)
     	buf += 8;
 	source = strsep(&buf, " ");
 	if (source == NULL) {
-	    screen_putf("Invalid $Search message: Missing source specification.\n");
+	    screen_putf(_("Invalid $Search message: Missing source specification.\n"));
 	    return;
 	}
 	if (!parse_search_selection(buf, &sel)) {
-	    screen_putf("Invalid $Search message: Invalid search specification.\n");
+	    screen_putf(_("Invalid $Search message: Invalid search specification.\n"));
 	    return;
 	}
 	if (strncmp(source, "Hub:", 4) == 0) {
 	    DCUserInfo *ui = hmap_get(hub_users, source+4);
 	    if (ui == NULL) {
-		screen_putf("Invalid $Search message: Unknown user %s.\n", source+4);
+		screen_putf(_("Invalid $Search message: Unknown user %s.\n"), source+4);
 		free(sel.patterns);
 		return;
 	    }
@@ -605,7 +604,7 @@ hub_handle_command(char *buf, uint32_t len)
 	} else {
             struct sockaddr_in addr;
     	    if (!parse_ip_and_port(source, &addr, DC_CLIENT_UDP_PORT)) {
-		screen_putf("Invalid $Search message: Invalid address specification.\n");
+		screen_putf(_("Invalid $Search message: Invalid address specification.\n"));
 		free(sel.patterns);
 		return;
 	    }
@@ -635,7 +634,7 @@ hub_input_available(void)
 
     res = byteq_read(hub_recvq, hub_socket);
     if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	warn_socket_eof(res, "receive from", "hub");
+    	warn_socket_error(res, false, _("hub"));
 	hub_disconnect();
 	return;
     }
@@ -644,7 +643,7 @@ hub_input_available(void)
         if (hub_recvq->buf[c] == '|') {
             /* Got a complete command. */
 	    if (c - start > 0)
-	    	dump_command("<--", hub_recvq->buf + start, c - start + 1);
+	    	dump_command(_("<--"), hub_recvq->buf + start, c - start + 1);
             hub_recvq->buf[c] = '\0'; /* Just to be on the safe side... */
             hub_handle_command(hub_recvq->buf + start, c - start);
             start = c+1;
@@ -668,24 +667,26 @@ hub_now_writable(void)
 	socklen_t addr_len;
 
         if (getsockopt(hub_socket, SOL_SOCKET, SO_ERROR, &error, &size) < 0) {
-            warn("Cannot get error status - %s\n", errstr);
+            warn(_("Cannot get error status - %s\n"), errstr);
 	    hub_disconnect();
 	    return;
         }
         if (error != 0) { /* The connect call on the socket failed. */
-	    warn("Cannot connect - %s\n", strerror(error) /* not errno! */);
+	    warn(_("Cannot connect - %s\n"), strerror(error) /* not errno! */);
 	    hub_disconnect();
 	    return;
 	}
 
 	addr_len = sizeof(local_addr);
 	if (getsockname(hub_socket, (struct sockaddr *) &local_addr, &addr_len) < 0) {
-    	    warn("Cannot get socket address - %s\n", errstr);
+	    warn(_("Cannot get socket address - %s\n"), errstr);
 	    hub_disconnect();
-            return;
+	    return;
 	}
+	if (force_listen_addr.s_addr != INADDR_NONE)
+	    local_addr.sin_addr = force_listen_addr;
 
-    	screen_putf("Connected to hub from %s.\n", sockaddr_in_str(&local_addr));
+    	screen_putf(_("Connected to hub from %s.\n"), sockaddr_in_str(&local_addr));
 
     	FD_CLR(hub_socket, &write_fds);
 	FD_SET(hub_socket, &read_fds);
@@ -697,7 +698,7 @@ hub_now_writable(void)
 	if (hub_sendq->cur > 0) {
 	    res = byteq_write(hub_sendq, hub_socket);
 	    if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	    	warn_socket_eof(res, "send to", "hub");
+    	    	warn_socket_error(res, true, _("hub"));
 		hub_disconnect();
 		return;
             }
@@ -725,7 +726,7 @@ hub_connect_user(DCUserInfo *ui)
 {
     if (is_active) {
     	if (ui->active_state == DC_ACTIVE_SENT_ACTIVE) {
-    	    warn("ConnectToMe already sent to user %s. Waiting.\n", ui->nick);
+    	    warn(_("ConnectToMe already sent to user %s. Waiting.\n"), ui->nick);
 	    return true;
 	}
     	if (!hub_putf("$ConnectToMe %s %s:%u|",
@@ -736,11 +737,11 @@ hub_connect_user(DCUserInfo *ui)
 	ui->active_state = DC_ACTIVE_SENT_ACTIVE;
     } else {
     	if (ui->active_state == DC_ACTIVE_SENT_PASSIVE) {
-    	    screen_putf("RevConnectToMe already sent to user %s. Waiting.\n", ui->nick);
+    	    screen_putf(_("RevConnectToMe already sent to user %s. Waiting.\n"), ui->nick);
 	    return true;
 	}
     	if (ui->active_state == DC_ACTIVE_RECEIVED_PASSIVE) {
-    	    screen_putf("User %s is also passive. Cannot communicate.\n", ui->nick);
+    	    screen_putf(_("User %s is also passive. Cannot communicate.\n"), ui->nick);
 	    return true;
 	}
     	if (!hub_putf("$RevConnectToMe %s %s|", my_nick, ui->nick))
