@@ -21,22 +21,32 @@
 #include <assert.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
-#include <signal.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/socket.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <inttypes.h>
-#include <fcntl.h>
+#include <signal.h>		/* POSIX */
+#include <stdlib.h>		/* C89 */
+#include <string.h>		/* C89 */
+#include <sys/socket.h>		/* POSIX */
+#include <sys/stat.h>		/* ? */
+#include <sys/types.h>		/* ? */
+#include <unistd.h>		/* POSIX */
+#include <inttypes.h>		/* ? */
+#include <fcntl.h>		/* ? */
+#include <getopt.h>		/* Gnulib/POSIX */
+#include "full-write.h"		/* Gnulib */
+#include "xvasprintf.h"		/* Gnulib */
+#include "version-etc.h"	/* Gnulib */
+#include "xalloc.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/error.h"
-#include "common/memory.h"
-#include "microdc.h"
-#include "full-write.h"
-#include "ipc.h"
-#include "xvasprintf.h"
 #include "common/strleftcmp.h"
+#include "ipc.h"
+#include "microdc.h"
+
+enum {
+    VERSION_OPT = 256,
+    HELP_OPT
+};
 
 #define LISTEN_QUEUE_CONNS 16
 
@@ -59,7 +69,8 @@ HMap *hub_users = NULL;
 HMap *pending_userinfo = NULL; 
 
 struct sockaddr_in local_addr;
-
+struct in_addr force_listen_addr = { INADDR_NONE };
+    
 /* settings/variables */
 int used_ul_slots = 0;
 int used_dl_slots = 0;
@@ -80,6 +91,7 @@ uint64_t my_share_size = 0;
 DCFileList *browse_list = NULL;
 DCUserInfo *browse_user = NULL;
 char *browse_path = NULL;
+bool browsing_myself = false;
 
 /* misc */
 uint16_t listen_port = 0;
@@ -91,6 +103,8 @@ PtrV *delete_dirs = NULL;
 static PtrV *search_udpmsg_out;	/* pending outgoing search results */
 uint32_t debug_level = 0;
 
+const char version_etc_copyright[] = "Copyright (C) 2004-2005 Oskar Liljeblad";
+
 static uint32_t user_conn_unknown_last = 0;
 static PtrV *user_conn_unknown_free = NULL;
 static ByteQ *search_recvq;
@@ -99,23 +113,37 @@ static int signal_pipe[2] = { -1, -1 };
 static int listen_socket = -1;
 static int search_socket = -1;
 
+static struct option long_opts[] = {
+    { "version", no_argument, NULL, VERSION_OPT },
+    { "help", no_argument, NULL, HELP_OPT },
+    { 0, 0, 0, 0 }
+};
+
 void
-warn_file_eof(int res, const char *op, const char *filename)
+warn_file_error(int res, bool write, const char *filename)
 {
-    if (res < 0) {
-	warn("%s: Cannot %s - %s\n", filename, op, errstr);
+    if (write) {
+	warn(_("%s: Cannot write to file - %s\n"), filename, errstr);
     } else {
-	warn("%s: Premature end of file\n", filename);
+	if (res < 0) {
+	    warn(_("%s: Cannot read from file - %s\n"), filename, errstr);
+	} else {
+	    warn(_("%s: Premature end of file\n"), filename);
+	}
     }
 }
 
 void
-warn_socket_eof(int res, const char *op, const char *subject)
+warn_socket_error(int res, bool write, const char *subject)
 {
-    if (res < 0) {
-	warn("Cannot %s %s - %s\n", op, subject, errstr);
+    if (write) {
+	warn("Cannot send to %s - %s\n", subject, errstr);
     } else {
-	warn("Disconnected from %s.\n", subject);
+	if (res < 0) {
+	    warn("Cannot receive from %s - %s\n", subject, errstr);
+	} else {
+	    warn("Disconnected from %s.\n", subject);
+	}
     }
 }
 
@@ -141,7 +169,7 @@ validate_nick(DCUserConn *uc, const char *nick)
 	    user_info_free(ui);
 	    return true;
 	}
-        screen_putf("No more connections to user %s allowed.\n", nick);
+        screen_putf(_("No more connections to user %s allowed.\n"), nick);
         hmap_remove(pending_userinfo, ui->nick);
 	user_info_free(ui);
 	return false;
@@ -155,7 +183,7 @@ validate_nick(DCUserConn *uc, const char *nick)
 	    uc->info->conn[uc->info->conn_count++] = uc;
 	    return true;
 	}
-        screen_putf("No more connections to user %s allowed.\n", nick);
+        screen_putf(_("No more connections to user %s allowed.\n"), nick);
     }
 
     return false;
@@ -185,8 +213,9 @@ validate_direction(DCUserConn *uc, DCTransferDirection dir)
 
     hmap_remove(user_conns, uc->name);
     ptrv_append(user_conn_unknown_free, uc->name);
-    newname = xasprintf("%s|%s", uc->info->nick, dir == DC_DIR_SEND ? "UL" : "DL");
-    screen_putf("User %s validated as %s\n", uc->name, newname);
+    newname = xasprintf(
+	    dir == DC_DIR_SEND ? _("%s|UL") : _("%s|DL"), uc->info->nick);
+    screen_putf(_("User %s validated as %s\n"), uc->name, newname);
     uc->name = newname;
     uc->dir = dir;
     hmap_put(user_conns, uc->name, uc);
@@ -210,14 +239,14 @@ handle_user_ipc_error(IPC *ipc, IPCError error, int result, void *userdata)
 
     switch (error) {
     case IPC_ERROR_WRITE:
-    	warn_socket_eof(result, "send to", "user process");
+    	warn_socket_error(result, true, _("user process"));
 	break;
     case IPC_ERROR_READ:
-    	warn_socket_eof(result, "receive from", "user process");
+    	warn_socket_error(result, false, _("user process"));
 	break;
     case IPC_ERROR_DATA:
     	//XXX: assert not reached
-    	screen_putf("Logical communication error with user process.\n");
+    	screen_putf(_("Logical communication error with user process.\n"));
     	break;
     }
     hmap_remove(user_conns, uc->name);
@@ -234,7 +263,7 @@ handle_user_msg(IPC *ipc, uint32_t id, void *userdata)
     	char *msg;
 
     	ipc_get(ipc, id, IPC_STR, &msg, IPC_END);
-    	screen_putf("User %s: %s", uc->name, msg);
+    	screen_putf(_("User %s: %s"), uc->name, msg);
 	free(msg);
     	return true;
     }
@@ -341,13 +370,13 @@ handle_user_msg(IPC *ipc, uint32_t id, void *userdata)
     	    uc->transfer_file = filename;
 	    uc->transfer_len = filesize;
 	    if (time(&uc->transfer_start) < 0) {
-	    	warn("Cannot get current time - %s\n", errstr);
+	    	warn(_("Cannot get current time - %s\n"), errstr);
 		uc->transfer_start = 0;
 	    }
 	}
-    //screen_putf("main pre put DC_MSG_TRANSFER_STARTING\n");
+	//screen_putf("main pre put DC_MSG_TRANSFER_STARTING\n");
 	reply = ipc_put(ipc, id, IPC_BOOL, reply, IPC_END);
-    //screen_putf("main post put DC_MSG_TRANSFER_STARTING\n");
+	//screen_putf("main post put DC_MSG_TRANSFER_STARTING\n");
 	return reply;
     }
     case DC_MSG_TRANSFER_FINISHED: {
@@ -368,7 +397,7 @@ handle_user_msg(IPC *ipc, uint32_t id, void *userdata)
 	    if (browse_user != NULL && strcmp(browse_user->nick, uc->info->nick) == 0 && browse_list == NULL) {
 		char *filename;
 
-		screen_putf("Now browsing %s.\n", browse_user->nick);
+		screen_putf(_("Now browsing %s.\n"), browse_user->nick);
 		filename = xasprintf("%s/%s.MyList.DcLst", listing_dir, browse_user->nick);
 		browse_list = filelist_open(filename);
 		browse_path = xstrdup("/");
@@ -383,7 +412,7 @@ handle_user_msg(IPC *ipc, uint32_t id, void *userdata)
     }
     case DC_MSG_DISCONNECT:
     default:
-        screen_putf("Received message %d from user process.\n", id);
+        screen_putf(_("Received message %d from user process.\n"), id);
     	hmap_remove(user_conns, uc->name);
 	user_disconnect(uc);
 	return false;
@@ -398,7 +427,7 @@ add_user_conn(DCUserConn *uc)
     if (user_conn_unknown_free->cur > 0) {
     	uc->name = ptrv_remove_first(user_conn_unknown_free);
     } else {
-	uc->name = xasprintf("unknown%d", user_conn_unknown_last+1);
+	uc->name = xasprintf(_("unknown%d"), user_conn_unknown_last+1);
 	user_conn_unknown_last++;
     }
     hmap_put(user_conns, uc->name, uc);
@@ -407,7 +436,7 @@ add_user_conn(DCUserConn *uc)
 void
 user_conn_cancel(DCUserConn *uc)
 {
-    warn("Disconnecting user.\n");
+    warn(_("Disconnecting user.\n"));
 
     if (!ipc_put(uc->ipc, DC_MSG_DISCONNECT, IPC_END))
     	return;
@@ -452,22 +481,30 @@ transfer_completion_generator(const char *base, int state)
 static void
 signal_received(int signal)
 {
-    if (full_write(signal_pipe[1], (char *) &signal, sizeof(int)) < sizeof(int))
-    	die("Cannot write - %s\n", errstr); /* die OK */
+    uint8_t signal_char;
+
+    signal_char = signal;
+    if (write(signal_pipe[1], &signal, sizeof(uint8_t)) < sizeof(uint8_t)) {
+	/* Die only if the signal is fatal. */
+	if (signal == SIGTERM || signal == SIGINT)
+	    die(_("Cannot write to signal pipe - %s\n"), errstr); /* die OK */
+	else
+	    warn(_("Cannot write to signal pipe - %s\n"), errstr);
+    }
 }
 
 static void
 read_signal_input(void)
 {
-    int signal;
+    uint8_t signal;
 
     /* This read is atomic since sizeof(int) < PIPE_BUF!
      * It also doesn't block since all data is already
      * available (otherwise select wouldn't tell us there
      * was data).
      */
-    if (read(signal_pipe[0], &signal, sizeof(int)) < 0) {
-	warn("Cannot read - %s\n", errstr);
+    if (read(signal_pipe[0], &signal, sizeof(uint8_t)) < 0) {
+	warn(_("Cannot read from signal pipe - %s\n"), errstr);
 	running = false;
 	return;
     }
@@ -495,7 +532,7 @@ run_script(const char *filename, bool allow_missing)
     file = fopen(filename, "r");
     if (file == NULL) {
     	if (!allow_missing || errno != ENOENT)
-    	    screen_putf("%s: Cannot open - %s\n", filename, errstr);
+    	    screen_putf(_("%s: Cannot open file - %s\n"), filename, errstr);
     	return;
     }
     while (getline(&line, &line_len, file) >= 0) {
@@ -506,13 +543,13 @@ run_script(const char *filename, bool allow_missing)
     }
     free(line);
     if (ferror(file)) {
-    	screen_putf("%s: Cannot read - %s\n", filename, errstr);
+    	screen_putf(_("%s: Cannot read from file - %s\n"), filename, errstr);
 	if (fclose(file) < 0)
-	    screen_putf("%s: Cannot close - %s\n", filename, errstr);
+	    screen_putf(_("%s: Cannot close file - %s\n"), filename, errstr);
 	return;
     }
     if (fclose(file) < 0)
-	screen_putf("%s: Cannot close - %s\n", filename, errstr);
+	screen_putf(_("%s: Cannot close file - %s\n"), filename, errstr);
 }
 
 bool
@@ -525,7 +562,7 @@ set_share_dir(const char *dir)
     if (hub_state >= DC_HUB_LOGGED_IN && !send_my_info())
     	return false;
     if (dir != NULL)
-    	screen_putf("Sharing %" PRId64 " bytes (%" PRId64 " MB) totally\n", my_share_size, my_share_size/(1024*1024));
+	screen_putf(_("Sharing %" PRId64 " bytes (%" PRId64 " MB) totally\n"), my_share_size, my_share_size/(1024*1024));
     return true;
 }
 
@@ -538,7 +575,7 @@ add_search_result(struct sockaddr_in *addr, char *results, uint32_t resultlen)
     if (search_udpmsg_out->cur == 0) {
     	res = sendto(search_socket, results, resultlen, 0, (struct sockaddr *) addr, sizeof(struct sockaddr_in));
         if (res == 0 || (res < 0 && errno != EAGAIN)) {
-	    warn_socket_eof(res, "send to", "user (search)");
+	    warn_socket_error(res, true, _("user (search result)"));
 	    return;
 	}
 	if (res == resultlen)
@@ -569,13 +606,14 @@ search_input_available(void)
     addrlen = sizeof(addr);
     res = byteq_recvfrom(search_recvq, search_socket, 0, (struct sockaddr *) &addr, &addrlen);
     if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	warn_socket_eof(res, "receive from", "user (search)");
+    	warn_socket_error(res, false, _("user (search result)"));
 	return;
     }
 
     if (search_recvq->buf[search_recvq->cur-1] != '|') {
         search_recvq->buf[search_recvq->cur-1] = '\0';
-    	screen_putf("Unterminated $SR in UDP packet, discarding.\n"/*, search_recvq->buf*/);
+    	screen_putf(_("Unterminated $SR in UDP packet, discarding.\n"));
+	/* XXX: dump package? */
     } else {
 	search_recvq->buf[search_recvq->cur-1] = '\0';
 	handle_search_result(search_recvq->buf, search_recvq->cur);
@@ -594,11 +632,11 @@ search_now_writable(void)
 	 */
     	res = sendto(search_socket, msg->data, msg->len, 0, (struct sockaddr *) &msg->addr, sizeof(struct sockaddr_in));
         if (res == 0 || (res < 0 && errno != EAGAIN))
-	    warn_socket_eof(res, "send to", "user (search)");
+	    warn_socket_error(res, true, _("user (search result)"));
 	if (res < 0 && errno == EAGAIN)
 	    break;
 
-    	dump_command("==>", msg->data, msg->len);
+    	dump_command(_("==>"), msg->data, msg->len);
 
 	ptrv_remove_first(search_udpmsg_out);
 	free(msg);
@@ -612,7 +650,7 @@ disable_active(void)
 {
     if (listen_socket >= 0) {
 	if (close(listen_socket) < 0)
-	    warn("Cannot close - %s\n", errstr);
+	    warn(_("Cannot close socket - %s\n"), errstr); /* XXX: (user connections server/listen socket) */
 	FD_CLR(listen_socket, &read_fds);
 	listen_socket = -1;
     }
@@ -627,18 +665,18 @@ enable_search(void)
 
     search_socket = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
     if (search_socket < 0) {
-	warn("Cannot create socket - %s\n", errstr);
+	warn(_("Cannot create socket - %s\n"), errstr);
 	return false;
     }
 
     if (fd_set_nonblock_flag(search_socket, true) < 0) {
-    	warn("Cannot set socket flags - %s\n", errstr);
+    	warn(_("Cannot set socket flags - %s\n"), errstr);
 	return false;
     }
 
     val = 1;
     if (setsockopt(search_socket, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) < 0)
-	warn("Cannot enable address reusing - %s\n", errstr);
+	warn(_("Cannot enable address reusing - %s\n"), errstr);
 
     if (listen_port != 0) {
     	addr_len = sizeof(addr);
@@ -646,7 +684,7 @@ enable_search(void)
 	addr.sin_port = htons(listen_port);
 	addr.sin_addr.s_addr = htonl(INADDR_ANY);
 	if (bind(search_socket, (struct sockaddr *) &addr, addr_len) < 0) {
-	    warn("Cannot bind to address - %s\n", errstr);
+	    warn(_("Cannot bind to address - %s\n"), errstr);
 	    return false;
 	}
     }
@@ -666,14 +704,14 @@ enable_active(uint16_t port)
     /* Create sockets. */
     listen_socket = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
     if (listen_socket < 0) {
-	warn("Cannot create socket - %s\n", errstr);
+	warn(_("Cannot create socket - %s\n"), errstr);
 	disable_active();
 	return false;
     }
 
     /* Set non-blocking */
     if (fd_set_nonblock_flag(listen_socket, true) < 0) {
-    	warn("Cannot set socket flags - %s\n", errstr);
+    	warn(_("Cannot set socket flags - %s\n"), errstr);
 	disable_active();
 	return false;
     }
@@ -681,7 +719,7 @@ enable_active(uint16_t port)
     /* Enable listen address reusing */
     val = 1;
     if (setsockopt(listen_socket, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) < 0)
-	warn("Cannot enable address reusing - %s\n", errstr);
+	warn(_("Cannot enable address reusing - %s\n"), errstr);
 
     /* Bind to address. If not binding, port is selected randomly. */
     addr_len = sizeof(addr);
@@ -690,7 +728,7 @@ enable_active(uint16_t port)
 	addr.sin_port = htons(port);
 	addr.sin_addr.s_addr = htonl(INADDR_ANY);
     	if (bind(listen_socket, (struct sockaddr *) &addr, addr_len) < 0) {
-	    warn("Cannot bind to address - %s\n", errstr);
+	    warn(_("Cannot bind to address - %s\n"), errstr);
 	    disable_active();
 	    return false;
 	}
@@ -698,19 +736,19 @@ enable_active(uint16_t port)
 
     /* Start listening for connections. */
     if (listen(listen_socket, LISTEN_QUEUE_CONNS) < 0) {
-	warn("Cannot listen - %s\n", errstr);
+	warn(_("Cannot listen - %s\n"), errstr);
 	disable_active();
 	return false;
     }
 
     /* Find port we are listening on. */
     if (getsockname(listen_socket, (struct sockaddr *) &addr, &addr_len) < 0) {
-    	warn("Cannot get socket address - %s\n", errstr);
+    	warn(_("Cannot get socket address - %s\n"), errstr);
 	disable_active();
 	return false;
     }
 
-    warn("Listening on %s.\n", sockaddr_in_str(&addr));
+    warn(_("Listening on %s.\n"), sockaddr_in_str(&addr));
     listen_port = ntohs(addr.sin_port);
     FD_SET(listen_socket, &read_fds);
     return true;
@@ -729,7 +767,7 @@ set_active(bool newactive, uint16_t port)
     /* Start of disable_search. */
     if (search_socket >= 0) {
 	if (close(search_socket) < 0)
-    	    warn("Cannot close - %s\n", errstr);
+    	    warn(_("Cannot close socket - %s\n"), errstr);
 	search_socket = -1;
     }
     /* End of disable_search. */
@@ -748,11 +786,11 @@ handle_listen_connection(void)
     addr_len = sizeof(addr);
     socket = accept(listen_socket, (struct sockaddr *) &addr, &addr_len);
     if (socket < 0) {
-	warn("Cannot accept connection - %s\n", errstr);
+	warn(_("Cannot accept user connection - %s\n"), errstr);
 	return;
     }
 
-    screen_putf("User from %s connected.\n", sockaddr_in_str(&addr));
+    screen_putf(_("User from %s connected.\n"), sockaddr_in_str(&addr));
     user_connection_new(&addr, socket);
 }
 
@@ -761,35 +799,55 @@ main (int argc, char **argv)
 {
     uint32_t c;
     char *filename;
+    char *tmpdir;
     struct sigaction sigact;
 
+    while (true) {
+	c = getopt_long(argc, argv, "", long_opts, NULL);
+	if (c == -1)
+	    break;
+
+	switch (c) {
+	case HELP_OPT:
+	    printf(_("Usage: %s [OPTION]...\n"), argv[0]);
+	    puts(_("Start microdc, a command-line based Direct Connect client.\n"));
+	    printf(_("      --help     display this help and exit\n"));
+	    printf(_("      --version  output version information and exit\n"));
+	    printf(_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+	    exit(EXIT_SUCCESS);
+	case VERSION_OPT:
+	    version_etc(stdout, NULL, PACKAGE, VERSION, "Oskar Liljeblad", NULL);
+	    exit(EXIT_SUCCESS);
+	}
+    }
+    
     if (pipe(signal_pipe) < 0) {
-    	warn("Cannot create pipe pair - %s\n", errstr);
+    	warn(_("Cannot create pipe pair - %s\n"), errstr);
 	goto cleanup;
     }
 
     sigact.sa_handler = signal_received;
     if (sigemptyset(&sigact.sa_mask) < 0) {
-        warn("Cannot empty signal set - %s\n", errstr);
+        warn(_("Cannot empty signal set - %s\n"), errstr);
 	goto cleanup;
     }
     sigact.sa_flags = SA_RESTART;
     sigact.sa_restorer = NULL;
     if (sigaction(SIGINT, &sigact, NULL) < 0) {
-        warn("Cannot register signal handler - %s\n", errstr);
+        warn(_("Cannot register signal handler - %s\n"), errstr);
 	goto cleanup;
     }
     if (sigaction(SIGTERM, &sigact, NULL) < 0) {
-        warn("Cannot register signal handler - %s\n", errstr);
+        warn(_("Cannot register signal handler - %s\n"), errstr);
 	goto cleanup;
     }
     if (sigaction(SIGUSR1, &sigact, NULL) < 0) {
-        warn("Cannot register signal handler - %s\n", errstr);
+        warn(_("Cannot register signal handler - %s\n"), errstr);
 	goto cleanup;
     }
     sigact.sa_handler = SIG_IGN;
     if (sigaction(SIGPIPE, &sigact, NULL) < 0) {
-        warn("Cannot register signal handler - %s\n", errstr);
+        warn(_("Cannot register signal handler - %s\n"), errstr);
 	goto cleanup;
     }
 
@@ -816,7 +874,14 @@ main (int argc, char **argv)
     my_speed = xstrdup("56Kbps");
     my_tag = xasprintf("%s V:%s", PACKAGE, VERSION);
     download_dir = xstrdup(".");
-    listing_dir = xasprintf("/tmp/%s.%d", PACKAGE, getpid());
+    tmpdir = get_temp_dir();
+    if (tmpdir == NULL) {
+	warn(_("Cannot find directory for temporary files - %s\n"), errstr);
+	goto cleanup;
+    }
+    filename = xasprintf("%s.%d", PACKAGE, getpid());
+    listing_dir = catfiles(tmpdir, filename);
+    free(filename);
     ptrv_append(delete_dirs, xstrdup(listing_dir));
     is_active = false;
     listen_port = 0;
@@ -842,7 +907,7 @@ main (int argc, char **argv)
 	res_write_fds = write_fds;
     	res = TEMP_FAILURE_RETRY(select(FD_SETSIZE, &res_read_fds, &res_write_fds, NULL, NULL));
 	if (res < 0) {
-	    warn("Cannot select - %s\n", errstr);
+	    warn(_("Cannot select - %s\n"), errstr);
 	    break;
 	}
 
@@ -915,12 +980,12 @@ cleanup:
 
 	    if (stat(filename, &st) < 0) {
 		if (errno != ENOENT)
-	    	    screen_putf("%s: Cannot get status - %s\n", filename, errstr);
+	    	    screen_putf(_("%s: Cannot get file status - %s\n"), filename, errstr);
 		free(filename);
 		continue;
 	    }
     	    if (unlink(filename) < 0)
-		screen_putf("%s: Cannot remove - %s\n", filename, errstr);
+		screen_putf(_("%s: Cannot remove file - %s\n"), filename, errstr);
 	    free(filename);
 	}
 	ptrv_free(delete_files);
@@ -933,25 +998,25 @@ cleanup:
 
 	    if (stat(filename, &st) < 0) {
 		if (errno != ENOENT)
-	    	    screen_putf("%s: Cannot get status - %s\n", filename, errstr);
+	    	    screen_putf(_("%s: Cannot get file status - %s\n"), filename, errstr);
 		free(filename);
 		continue;
 	    }
     	    if (rmdir(filename) < 0)
-		screen_putf("%s: Cannot remove - %s\n", filename, errstr);
+		screen_putf(_("%s: Cannot remove file - %s\n"), filename, errstr);
 	    free(filename);
 	}
 	ptrv_free(delete_dirs);
     }
 
     if (search_socket >= 0 && close(search_socket) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close search results socket - %s\n"), errstr);
     if (listen_socket >= 0 && close(listen_socket) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close user connections socket - %s\n"), errstr);
     if (signal_pipe[0] >= 0 && close(signal_pipe[0]) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close signal pipe - %s\n"), errstr);
     if (signal_pipe[1] >= 0 && close(signal_pipe[1]) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close signal pipe - %s\n"), errstr);
 
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
