@@ -158,6 +158,7 @@ struct _DCSearchSelection {
 
 struct _DCSearchRequest {
     DCSearchSelection selection;
+    time_t issue_time;
     PtrV *responses;
 };
 
@@ -245,6 +246,7 @@ extern ByteQ *hub_sendq;
 extern HMap *hub_users;
 extern HMap *user_conns;
 extern struct sockaddr_in local_addr;
+extern struct in_addr force_listen_addr;
 extern int hub_socket;
 extern bool running;
 extern HMap *pending_userinfo;
@@ -271,7 +273,8 @@ extern int used_dl_slots;
 extern uint32_t debug_level;
 
 extern DCFileList *browse_list; /* list of user we browse */
-extern DCUserInfo *browse_user; /* user we browse OR WISH to browse */
+extern DCUserInfo *browse_user; /* user we browse OR WISH to browse, NULL if possibly browsing ourself */
+extern bool browsing_myself;
 extern char *browse_path;
 
 /* hub.c */
@@ -290,6 +293,7 @@ char *speed_completion_generator(const char *base, int state);
 extern struct sockaddr_in hub_addr;
 extern char *hub_name;
 bool say_user_completion_generator(const char *base, int state, DCCompletionEntry *ce);
+char *user_or_myself_completion_generator(const char *base, int state);
 
 /* command.c */
 void command_execute(char *line);
@@ -328,8 +332,8 @@ uint32_t get_user_conn_count(DCUserInfo *info);
 bool get_package_file(const char *name, char **outname);
 char *transfer_completion_generator(const char *base, int state); /* XXX: move transfer.c? */
 void user_conn_cancel(DCUserConn *uc);
-void warn_file_eof(int res, const char *op, const char *filename);
-void warn_socket_eof(int res, const char *op, const char *subject);
+void warn_file_error(int res, bool write, const char *filename);
+void warn_socket_error(int res, bool write, const char *subject);
 void add_search_result(struct sockaddr_in *addr, char *results, uint32_t resultlen);
 void free_queued_file(DCQueuedFile *qf); /* XXX: move transfer.c? */
 
@@ -368,6 +372,9 @@ char *getenv_default(const char *name, char *defvalue);
 #define COMPARE_RETURN(a,b) { if ((a) < (b)) return -1; if ((a) > (b)) return 1; }
 #define COMPARE_RETURN_FUNC(f) { int _c = (f); if (_c != 0) return _c; }
 char *sockaddr_in_str(struct sockaddr_in *addr);
+bool direxists(const char *dir);
+char *get_temp_dir(void);
+bool parse_ip_and_port(char *source, struct sockaddr_in *addr, uint16_t defport);
 
 /* search.c */
 bool parse_search_selection(char *str, DCSearchSelection *data);
