@@ -18,28 +18,32 @@
  */
 
 #include <config.h>
-#include <assert.h>
-#include <netinet/in.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <signal.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <inttypes.h>
-#include "xstrndup.h"
-#include "microdc.h"
-#include "common/memory.h"
+#include <assert.h>		/* ? */
+#include <netinet/in.h>		/* ? */
+#include <unistd.h>		/* POSIX */
+#include <sys/types.h>		/* ? */
+#include <sys/stat.h>		/* ? */
+#include <sys/wait.h>		/* ? */
+#include <fcntl.h>		/* ? */
+#include <ctype.h>		/* C89 */
+#include <signal.h>		/* ? */
+#include <sys/socket.h>		/* ? */
+#include <netinet/in.h>		/* ? */
+#include <arpa/inet.h>		/* ? */
+#include <inttypes.h>		/* ? */
+#include "dirname.h"		/* Gnulib */
+#include "full-write.h"		/* Gnulib */
+#include "xstrndup.h"		/* Gnulib */
+#include "xvasprintf.h"		/* Gnulib */
+#include "xalloc.h"		/* Gnulib */
+#include "minmax.h"		/* Gnulib */
+#include "gettext.h"		/* Gnulib/GNU gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "common/error.h"
 #include "common/intparse.h"
-#include "full-write.h"
 #include "common/common.h"
-#include "xvasprintf.h"
-#include "dirname.h"
+#include "microdc.h"
 
 #define LOCK_STRING "ABCABCABCABCABCABCABCABCABCABCABCA"
 #define LOCK_STRING_LEN (sizeof(LOCK_STRING)-1)
@@ -83,7 +87,7 @@ static bool
 check_state(char *buf, DCUserState state)
 {
     if (user_state != state) {
-	warn("Received %s message in wrong state.\n", strtok(buf, " "));
+	warn(_("Received %s message in wrong state.\n"), strtok(buf, " "));
 	user_running = false;
 	return false;
     }
@@ -109,7 +113,7 @@ handle_main_msg(IPC *ipc, uint32_t id, void *userdata)
     //	ipc_get(ipc, id, IPC_END);
     //	return ipc_put(ipc, id, IPC_INT64, transfer_pos, IPC_END);
     default:
-        screen_putf("Received message %d from main process.\n", id);
+        screen_putf(_("Received message %d from main process.\n"), id);
 	user_running = false;
         return false;
     }
@@ -122,14 +126,14 @@ handle_main_ipc_error(IPC *ipc, IPCError error, int result, void *userdata)
     screen_writer = default_warn_writer;
     switch (error) {
     case IPC_ERROR_WRITE:
-    	warn_socket_eof(result, "send to", "main process");
+    	warn_socket_error(result, true, _("main process"));
 	break;
     case IPC_ERROR_READ:
-    	warn_socket_eof(result, "receive from", "main process");
+    	warn_socket_error(result, false, _("main process"));
 	break;
     case IPC_ERROR_DATA:
     	//XXX: assert not reached
-    	screen_putf("Logical communication error with main process.\n");
+    	screen_putf(_("Logical communication error with main process.\n"));
     	break;
     }
     user_running = false;
@@ -170,7 +174,7 @@ user_putf(const char *format, ...)
     va_end(args);
 
     if (res < 0) {
-    	warn("Cannot append to user sendq - %s\n", errstr);
+    	warn(_("Cannot append to user send queue - %s\n"), errstr);
 	user_running = false;
 	return false;
     }
@@ -180,7 +184,7 @@ user_putf(const char *format, ...)
 
     res = byteq_write(user_sendq, user_socket);
     if (res == 0 || (res < 0 && errno != EAGAIN)) {
-	warn_socket_eof(res, "send to", "user");
+	warn_socket_error(res, true, _("user"));
 	user_running = false;
 	return false;
     }
@@ -203,7 +207,7 @@ download_next_file(void)
     if (!ipc_get(ipc, DC_MSG_NEXT_DOWNLOAD, IPC_STR, &dl_file, IPC_STR, &dl_base, IPC_INT32, &file_flag, IPC_END))
     	return false;
     if (dl_file == NULL) {
-        screen_putf("No more files to download.\n");
+        screen_putf(_("No more files to download.\n"));
         user_running = false;
 	return false;
     }
@@ -245,7 +249,7 @@ direction_validate(DCTransferDirection dir)
     if (!ipc_get(ipc, DC_MSG_VALIDATE_DIR, IPC_BOOL, &reply, IPC_END))
     	return false;
     if (!reply) {
-	warn("Too many connections with user, or no free slots.\n");
+	warn(_("Too many connections to user, or no free slots.\n"));
 	user_running = false;
 	return false;
     }
@@ -262,7 +266,7 @@ nick_validate(const char *str)
     if (!ipc_get(ipc, DC_MSG_VALIDATE_NICK, IPC_BOOL, &reply, IPC_END))
     	return false;
     if (!reply) {
-	screen_putf("User %s not on hub, or too many connections to user.\n", str);
+	screen_putf(_("User %s not on hub, or too many connections to user.\n"), str);
 	user_running = false;
     	return false;
     }
@@ -284,30 +288,30 @@ open_download_file(void)
     }
 
     if (file_flag == DC_TF_NORMAL) {
-        char *tmp = xasprintf("%s.part", localfile);
+        char *tmp = xasprintf(_("%s.part"), localfile);
         free(localfile);
         localfile = tmp;
     }
 
     transfer_file/*DL*/ = open/*64*/(localfile, O_CREAT|O_EXCL|O_WRONLY, 0644);
     if (transfer_file/*DL*/ < 0) {
-	warn("%s: Cannot open file for writing - %s\n", localfile, errstr);
+	warn(_("%s: Cannot open file for writing - %s\n"), localfile, errstr);
 	//free(localfile); localfile = NULL;
 	//free(file_name/*DL*/); file_name/*DL*/ = NULL;
 	return download_next_file();
     }
 
     if (file_size == 0) {
-	warn("File %s is empty, not starting download.\n", file_name/*DL*/);
+	warn(_("File %s is empty, not starting download.\n"), file_name/*DL*/);
 	if (close(transfer_file/*DL*/) < 0)
-	    warn("%s: Cannot close - %s\n", file_name/*DL*/, errstr);
+	    warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
 	transfer_file/*DL*/ = -1;
 	return download_next_file();
     }
 
     if (!user_putf("$Send|"))
 	return false; /* maybe free file_name here? */
-    warn("Starting download for file %s, %" PRIu64 " bytes\n", file_name/*DL*/, file_size);
+    warn(_("Starting download for file %s, %" PRIu64 " bytes\n"), file_name/*DL*/, file_size);
     if (!ipc_put(ipc, DC_MSG_TRANSFER_STARTING/*DL*/, IPC_INT64, file_size, IPC_STR, file_name/*DL*/, IPC_END))
 	return false;
     if (!ipc_get(ipc, DC_MSG_TRANSFER_STARTING/*DL*/, IPC_BOOL, &reply, IPC_END))
@@ -352,7 +356,7 @@ open_upload_file(const char *str, uint64_t offset)
     	return false;
 
     if (uploadfile == NULL) {
-        screen_putf("Cannot resolve upload file (invalid filename or no sharedir)\n");
+        screen_putf(_("Cannot resolve upload file - invalid filename or no sharedir\n"));
 	if (!user_putf("$Error File Not Available|"))
 	    return false;
 	user_running = false;
@@ -364,14 +368,14 @@ open_upload_file(const char *str, uint64_t offset)
 
     /* Get file status and check offset. */
     if (stat(file_name/*UL*/, &st) < 0) {
-        screen_putf("%s: Cannot get status - %s\n", file_name/*UL*/, errstr);
+        screen_putf(_("%s: Cannot get file status - %s\n"), file_name/*UL*/, errstr);
 	//free(file_name/*UL*/); file_name/*UL*/ = NULL;
 	if (!user_putf("$Error File Not Available|"))
 	    return false;
         return true;
     }
     if (offset > st.st_size) {
-	screen_putf("%s: Offset %" PRId64 " out of range\n", file_name/*UL*/, offset);
+	screen_putf(_("%s: Offset %" PRId64 " out of range\n"), file_name/*UL*/, offset);
 	//free(file_name/*UL*/); file_name/*UL*/ = NULL;
 	if (!user_putf("$Error Offset out of range|"))
 	    return false;
@@ -381,18 +385,18 @@ open_upload_file(const char *str, uint64_t offset)
     /* Open file and seek to desired position. */
     transfer_file/*UL*/ = open/*64*/(file_name/*UL*/, O_RDONLY);
     if (transfer_file/*UL*/ < 0) {
-	screen_putf("%s: Cannot open file for reading - %s\n", file_name/*UL*/, errstr);
+	screen_putf(_("%s: Cannot open file for reading - %s\n"), file_name/*UL*/, errstr);
 	//free(file_name/*UL*/); file_name/*UL*/ = NULL;
 	if (!user_putf("$Error File Not Available|"))
 	    return false;
         return true;
     }
     if (offset != 0 && lseek/*64*/(transfer_file/*UL*/, offset, SEEK_SET) < 0) {
-    	screen_putf("%s: Cannot seek - %s\n", file_name/*UL*/, errstr);
+    	screen_putf(_("%s: Cannot seek in file - %s\n"), file_name/*UL*/, errstr);
 	if (!user_putf("$Error File Not Available|"))
 	    return false;
 	if (close(transfer_file/*UL*/) < 0)
-	    warn("%s: Cannot close - %s\n", file_name/*UL*/, errstr);
+	    warn(_("%s: Cannot close file - %s\n"), file_name/*UL*/, errstr);
 	transfer_file/*UL*/ = -1;
 	//free(file_name/*UL*/); file_name/*UL*/ = NULL;
 	return true;
@@ -445,9 +449,9 @@ user_handle_command(char *buf, uint32_t len)
 
     	res = full_write(transfer_file/*DL*/, buf, len);
 	if (res < len) {
-	    warn_file_eof(res, "write", file_name/*DL*/);
+	    warn_file_error(res, true, file_name/*DL*/);
 	    if (close(transfer_file/*DL*/) < 0)
-	    	warn("%s: Cannot close - %s\n", file_name/*DL*/, errstr);
+	    	warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
 	    transfer_file/*DL*/ = -1;
 	    //free(localfile); localfile = NULL;
 	    //free(file_name/*DL*/); file_name/*DL*/ = NULL;
@@ -460,14 +464,14 @@ user_handle_command(char *buf, uint32_t len)
 	data_size/*DL*/ -= len;
     	send_user_status(file_pos);
     	if (file_pos == file_size) {
-	    screen_putf("%s: Download complete.\n", base_name(file_name/*DL*/));
+	    screen_putf(_("%s: Download complete.\n"), base_name(file_name/*DL*/));
 	    if (close(transfer_file/*DL*/) < 0)
-	    	warn("%s: Cannot close - %s\n", file_name/*DL*/, errstr);
+	    	warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
             if (file_flag == DC_TF_NORMAL) {
                 char *tmp;
                 tmp = xstrndup(localfile, strlen(localfile)-5);
                 if (rename(localfile, tmp) < 0)
-                    warn("%s: Cannot rename to `%s' - %s\n", localfile, tmp, errstr);
+                    warn(_("%s: Cannot rename file to `%s' - %s\n"), localfile, tmp, errstr);
                 free(tmp);
             }
             //free(localfile); localfile = NULL;
@@ -501,7 +505,7 @@ user_handle_command(char *buf, uint32_t len)
 	    return;
         key = memmem(buf+6, len-6, " Pk=", 4);
         if (key == NULL) {
-            screen_putf("Invalid $Lock message: Missing Pk value");
+            screen_putf(_("Invalid $Lock message: Missing Pk value\n"));
 	    key = buf+len;
 	}
         key = decode_lock(buf+6, key-buf-6, DC_CLIENT_BASE_KEY);
@@ -529,7 +533,7 @@ user_handle_command(char *buf, uint32_t len)
 
     	token = strtok(buf+11, " ");
 	if (token == NULL) {
-	    warn("Invalid $Direction message: Missing direction string\n");
+	    warn(_("Invalid $Direction message: Missing direction parameter\n"));
 	    user_running = false;
 	    return;
 	}
@@ -538,19 +542,19 @@ user_handle_command(char *buf, uint32_t len)
 	} else if (strcmp(token, "Download") == 0) {
 	    they_download = true;
 	} else {
-	    warn("Invalid $Direction message: Invalid direction string\n");
+	    warn(_("Invalid $Direction message: Invalid direction parameter\n"));
 	    user_running = false;
 	    return;
 	}
 
 	token = strtok(NULL, " ");
 	if (token == NULL) {
-	    screen_putf("Invalid $Direction message: Missing direction integer\n");
+	    screen_putf(_("Invalid $Direction message: Missing challenge parameter\n"));
 	    user_running = false;
 	    return;
 	}
 	if (!parse_uint16(token, &remote_rand)) {
-	    warn("Invalid $Direction message: Invalid direction integer\n");
+	    warn(_("Invalid $Direction message: Invalid challenge parameter\n"));
 	    user_running = false;
 	    return;
 	}
@@ -575,7 +579,7 @@ user_handle_command(char *buf, uint32_t len)
 	} else {
 	    // Remote wants to upload. Do we want to upload too?
 	    if (!we_download) {
-		warn("User does not want to download, nor do we.\n");
+		warn(_("User does not want to download, nor do we.\n"));
 		user_running = false;
 		return;
 	    }
@@ -592,7 +596,7 @@ user_handle_command(char *buf, uint32_t len)
 	    return;
     	key = decode_lock(LOCK_STRING, LOCK_STRING_LEN, DC_CLIENT_BASE_KEY);
 	if (strcmp(buf+5, key) != 0)
-	    screen_putf("Invalid $Key message: Incorrect key, ignoring\n");
+	    screen_putf(_("Invalid $Key message: Incorrect key, ignoring\n"));
 	free(key);
     	if (our_dir == DC_DIR_SEND) {
 	    user_state = DC_USER_GET;
@@ -605,18 +609,18 @@ user_handle_command(char *buf, uint32_t len)
 	uint64_t offset;
 
 	if (user_state != DC_USER_SEND_GET && user_state != DC_USER_GET) {
-	    warn("Received %s message in wrong state.\n", strtok(buf, " "));
+	    warn(_("Received %s message in wrong state.\n"), strtok(buf, " "));
 	    user_running = false;
 	    return;
 	}
 	token = strtok(buf+5, "$");
 	if (token == NULL) {
-	    screen_putf("Invalid $Get message: Missing offset, assuming start\n");
+	    screen_putf(_("Invalid $Get message: Missing offset, assuming start\n"));
 	    offset = 0;
 	} else {
 	    token = strtok(NULL, ""); /* Cannot fail! */
 	    if (!parse_uint64(token, &offset)) {
-	    	screen_putf("Invalid $Get message: Offset not integer\n");
+	    	screen_putf(_("Invalid $Get message: Offset not integer\n"));
 		user_running = false;
 		return;
 	    }
@@ -631,7 +635,7 @@ user_handle_command(char *buf, uint32_t len)
 	    if (!ipc_put(ipc, DC_MSG_TRANSFER_FINISHED/*UL*/, IPC_INT32, 0, IPC_END))
 	    	return;
 	    if (close(transfer_file/*UL*/) < 0)
-		warn("%s: Cannot close - %s\n", file_name/*UL*/, errstr);
+		warn(_("%s: Cannot close file - %s\n"), file_name/*UL*/, errstr);
 	    transfer_file/*UL*/ = -1;
 	}
     	open_upload_file(buf+5, offset); /* Changes state */
@@ -639,14 +643,14 @@ user_handle_command(char *buf, uint32_t len)
     else if (len == 9 && strcmp(buf, "$MaxedOut") == 0) {
     	if (!check_state(buf, DC_USER_FILE_LENGTH))
 	    return;
-	screen_putf("User is maxed out.\n");
+	screen_putf(_("User is maxed out.\n"));
 	user_running = false;
     }
     else if (len >= 12 && strncmp(buf, "$FileLength ", 12) == 0) {
     	if (!check_state(buf, DC_USER_FILE_LENGTH))
 	    return;
 	if (!parse_uint64(buf+12, &file_size)) {
-	    warn("Invalid $FileLength message: Invalid length integer\n");
+	    warn(_("Invalid $FileLength message: Invalid length integer\n"));
 	    user_running = false;
 	    return;
 	}
@@ -657,14 +661,14 @@ user_handle_command(char *buf, uint32_t len)
     	    if (strcmp(buf+7, "File Not Available") == 0) {
 	    	screen_putf("%s: File not available on remote\n", file_name/*DL*/);
 		if (close(transfer_file/*DL*/) < 0)
-	    	    warn("%s: Cannot close - %s\n", file_name/*DL*/, errstr);
+	    	    warn(_("%s: Cannot close file - %s\n"), file_name/*DL*/, errstr);
 		// free(file_name/*DL*/); file_name/*UL*/ = NULL;
 		transfer_file/*DL*/ = -1;
 		download_next_file();
 	    }
 	    return;
 	}
-	screen_putf("Received error from user: %s\n", buf+7);
+	screen_putf(_("Received error from user: %s\n"), buf+7);
 	user_running = false;
     }
     else if (len >= 5 && strncmp(buf, "$Send ", 5) == 0) {
@@ -694,7 +698,7 @@ user_input_available(void)
 
     res = byteq_read(user_recvq, user_socket);
     if (res == 0 || (res < 0 && errno != EAGAIN)) {
- 	warn_socket_eof(res, "receive from", "user");
+ 	warn_socket_error(res, false, _("user"));
 	user_running = false;
 	return;
     }
@@ -704,7 +708,7 @@ user_input_available(void)
 	    uint32_t size;
 
 	    /* Handle what've got, but never more than data_size */
-	    size = min(data_size/*DL*/, user_recvq->cur - start);
+	    size = MIN(data_size/*DL*/, user_recvq->cur - start);
     	    user_handle_command(user_recvq->buf + start, size);
 	    start += size;
 	    if (!user_running)
@@ -714,7 +718,7 @@ user_input_available(void)
         else if (user_recvq->buf[c] == '|') {
             /* Got a complete command. */
 	    if (c - start > 0)
-	    	dump_command("<--", user_recvq->buf + start, c - start + 1);
+	    	dump_command(_("<--"), user_recvq->buf + start, c - start + 1);
             user_recvq->buf[c] = '\0'; /* Just to be on the safe side... */
             user_handle_command(user_recvq->buf + start, c - start);
             start = c+1;
@@ -737,12 +741,12 @@ user_now_writable(void)
         int size = sizeof(error);
 
         if (getsockopt(user_socket, SOL_SOCKET, SO_ERROR, &error, &size) < 0) {
-            warn("Cannot get error status - %s\n", errstr);
+            warn(_("Cannot get error status - %s\n"), errstr);
 	    user_running = false;
 	    return;
         }
         if (error != 0) { /* The connect call on the socket failed. */
-	    warn("Cannot connect - %s\n", strerror(error) /* not errno! */);
+	    warn(_("Cannot connect - %s\n"), strerror(error) /* not errno! */);
 	    user_running = false;
 	    return;
 	}
@@ -752,7 +756,7 @@ user_now_writable(void)
 	user_state = DC_USER_MYNICK;
 
     	if (we_connected) {
-	    screen_putf("Connected to user.\n");
+	    screen_putf(_("Connected to user.\n"));
 	    if (!user_putf("$MyNick %s|", my_nick))
 		return;
     	    if (!user_putf("$Lock %s Pk=%s|", LOCK_STRING, LOCK_PK_STRING))
@@ -764,13 +768,13 @@ user_now_writable(void)
     	ssize_t res;
 
     	assert(file_size != 0);
-    	block = min(DEFAULT_SENDQ_SIZE, file_size-file_pos);
+    	block = MIN(DEFAULT_SENDQ_SIZE, file_size-file_pos);
 	if (block > 0 && user_sendq->cur == 0) { //if (user_sendq->cur < user_sendq->max) {
 	    res = byteq_full_read_upto(user_sendq, transfer_file/*UL*/, block);
 	    if (res < block) {
-	    	warn_file_eof(res, "read", file_name/*UL*/);
+	    	warn_file_error(res, false, file_name/*UL*/);
 		if (close(transfer_file/*UL*/) < 0)
-		    warn("%s: Cannot close - %s\n", file_name/*UL*/, errstr);
+		    warn(_("%s: Cannot close file - %s\n"), file_name/*UL*/, errstr);
 		transfer_file/*UL*/ = -1;
 		//free(file_name/*UL*/); file_name/*UL*/ = NULL;
 		user_running = false;
@@ -782,7 +786,7 @@ user_now_writable(void)
     	assert(user_sendq->cur != 0);
 	res = byteq_write(user_sendq, user_socket);
 	if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	    warn_socket_eof(res, "write to", "user");
+    	    warn_socket_error(res, true, _("user"));
 	    user_running = false;
 	    return;
 	}
@@ -790,10 +794,10 @@ user_now_writable(void)
   	send_user_status(file_pos - user_sendq->cur);
 
 	if (file_pos == file_size && user_sendq->cur == 0) {
-	    screen_putf("%s: Upload complete.\n", base_name(file_name/*UL*/));
+	    screen_putf(_("%s: Upload complete.\n"), base_name(file_name/*UL*/));
     	    FD_CLR(user_socket, &user_write_fds);
 	    if (close(transfer_file/*UL*/) < 0)
-	    	warn("Cannot close - %s\n", errstr);
+	    	warn(_("%s: Cannot close file - %s\n"), file_name/*UL*/, errstr);
 	    transfer_file = -1;
     	    free(file_name/*UL*/); file_name/*UL*/ = NULL;
 	    if (!ipc_put(ipc, DC_MSG_TRANSFER_FINISHED/*UL*/, IPC_INT32, 0, IPC_END))
@@ -807,7 +811,7 @@ user_now_writable(void)
 	if (user_sendq->cur > 0) {
 	    res = byteq_write(user_sendq, user_socket);
 	    if (res == 0 || (res < 0 && errno != EAGAIN)) {
-    	    	warn_socket_eof(res, "write to", "user");
+    	    	warn_socket_error(res, true, _("user"));
 	    	user_running = false;
 		return;
             }
@@ -820,23 +824,31 @@ user_now_writable(void)
 static void
 signal_received(int signal)
 {
+    uint8_t signal_char;
+
+    signal_char = signal;
     /* We don't care if this blocks - since we can't postpone this. */
-    if (write(signal_pipe[1], (char *) &signal, sizeof(int)) < sizeof(int))
-    	die("Cannot write - %s\n", errstr); /* die OK */
+    if (write(signal_pipe[1], &signal_char, sizeof(uint8_t)) < sizeof(uint8_t)) {
+	/* Die only if the signal is fatal. */
+	if (signal == SIGTERM || signal == SIGINT)
+	    die(_("Cannot write to signal pipe - %s\n"), errstr); /* die OK */
+	else
+	    warn(_("Cannot write to signal pipe - %s\n"), errstr);
+    }
 }
 
 static void
 read_signal_input(void)
 {
-    int signal;
+    uint8_t signal;
 
     /* This read is atomic since sizeof(int) < PIPE_BUF!
      * It also doesn't block since all data is already
      * available (otherwise select wouldn't tell us there
      * was data).
      */
-    if (read(signal_pipe[0], &signal, sizeof(int)) < 0) {
-	warn("read failed - %s\n", errstr);
+    if (read(signal_pipe[0], &signal, sizeof(uint8_t)) < 0) {
+	warn(_("Cannot read from signal pipe - %s\n"), errstr);
 	user_running = false;
 	return;
     }
@@ -861,21 +873,21 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
     warn_writer = user_screen_vputf;
 
     if (close(main_sockets[0]) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close socket - %s\n"), errstr);
 
     if (pipe(signal_pipe) < 0) {
-    	warn("Cannot create pipe pair - %s\n", errstr);
+    	warn(_("Cannot create pipe pair - %s\n"), errstr);
 	goto cleanup;
     }
     sigact.sa_handler = signal_received;
     if (sigemptyset(&sigact.sa_mask) < 0) {
-        warn("Cannot empty signal set - %s\n", errstr);
+        warn(_("Cannot empty signal set - %s\n"), errstr);
 	goto cleanup;
     }
     sigact.sa_flags = SA_RESTART;
     sigact.sa_restorer = NULL;
     if (sigaction(SIGTERM, &sigact, NULL) < 0 || sigaction(SIGUSR1, &sigact, NULL) < 0) {
-        warn("Cannot register signal handler - %s\n", errstr);
+        warn(_("Cannot register signal handler - %s\n"), errstr);
 	goto cleanup;
     }
 
@@ -890,7 +902,7 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
     if (sock < 0) {
 	user_socket = socket(PF_INET, SOCK_STREAM, 0);
 	if (user_socket < 0) {
-            warn("Cannot create socket - %s\n", errstr);
+            warn(_("Cannot create socket - %s\n"), errstr);
 	    goto cleanup;
 	}
     	we_connected = true;
@@ -901,7 +913,7 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
 
     /* Set non-blocking I/O on socket. */
     if (fd_set_nonblock_flag(user_socket, true) < 0) {
-        warn("Cannot set socket flags - %s\n", errstr);
+        warn(_("Cannot set socket flags - %s\n"), errstr);
         goto cleanup;
     }
 
@@ -912,7 +924,7 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
 	 */
 	if (connect(user_socket, (struct sockaddr *) addr, sizeof(struct sockaddr_in)) < 0
 	    	&& errno != EINPROGRESS) {
-            warn("Cannot connect - %s\n", errstr);
+            warn(_("Cannot connect - %s\n"), errstr);
 	    goto cleanup;
 	}
     }
@@ -930,7 +942,7 @@ user_main(int main_sockets[2], struct sockaddr_in *addr, int sock)
 	res_read_fds = user_read_fds;
 	res_write_fds = user_write_fds;
 	if (TEMP_FAILURE_RETRY(select(FD_SETSIZE, &res_read_fds, &res_write_fds, NULL, NULL)) < 0) {
-	    warn("Cannot select - %s\n", errstr);
+	    warn(_("Cannot select - %s\n"), errstr);
 	    break;
 	}
 
@@ -959,17 +971,17 @@ cleanup:
     byteq_free(user_sendq);
 
     if (transfer_file >= 0 && close(transfer_file) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close transfer file - %s\n"), errstr); /* XXX: should print WHAT file - then remove " transfer" */
     if (signal_pipe[0] >= 0 && close(signal_pipe[0]) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close signal pipe - %s\n"), errstr);
     if (signal_pipe[1] >= 0 && close(signal_pipe[1]) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close signal pipe - %s\n"), errstr);
     if (main_socket >= 0 && close(main_socket) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close main process socket - %s\n"), errstr);	/* XXX: "socket" or "connection" */
     if (user_socket >= 0 && close(user_socket) < 0)
-    	warn("Cannot close - %s\n", errstr);
+    	warn(_("Cannot close user connection - %s\n"), errstr);
 
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
 
 DCUserConn *
@@ -980,26 +992,26 @@ user_connection_new(struct sockaddr_in *addr, int user_socket)
     pid_t pid;
 
     if (socketpair(AF_LOCAL, SOCK_STREAM, 0, main_sockets) < 0) {
-    	screen_putf("Cannot create socket pair - %s\n", errstr);
+    	screen_putf(_("Cannot create socket pair - %s\n"), errstr);
 	return NULL;
     }
 
     pid = fork();
     if (pid < 0) {
-    	screen_putf("Cannot create process - %s\n", errstr);
+    	screen_putf(_("Cannot create process - %s\n"), errstr);
 	if (close(main_sockets[0]) < 0)
-	    screen_putf("Cannot close - %s\n", errstr);
+	    screen_putf(_("Cannot close socket - %s\n"), errstr);
 	if (close(main_sockets[1]) < 0)
-	    screen_putf("Cannot close - %s\n", errstr);
+	    screen_putf(_("Cannot close socket - %s\n"), errstr);
 	return NULL;
     }
     if (pid == 0)
 	user_main(main_sockets, addr, user_socket);
 
     if (close(main_sockets[1]) < 0)
-    	screen_putf("Cannot close - %s\n", errstr);
+    	screen_putf(_("Cannot close socket - %s\n"), errstr);
     if (user_socket >= 0 && close(user_socket) < 0)
-    	screen_putf("Cannot close - %s\n", errstr);
+    	screen_putf(_("Cannot close socket - %s\n"), errstr);
 
     uc = xmalloc(sizeof(DCUserConn));
     uc->pid = pid;
@@ -1020,7 +1032,7 @@ user_disconnect(DCUserConn *uc)
 {
     int status;
 
-    screen_putf("User %s: Shutting down user connection.\n", uc->name);
+    screen_putf(_("User %s: Shutting down user connection.\n"), uc->name);
     ipc_free(uc->ipc);
     if (uc->info != NULL) {
     	int c;
@@ -1045,12 +1057,12 @@ user_disconnect(DCUserConn *uc)
     FD_CLR(uc->main_socket, &read_fds);
     FD_CLR(uc->main_socket, &write_fds);
     if (close(uc->main_socket) < 0)
-	warn("Cannot close - %s\n", errstr);
+	warn(_("Cannot close socket - %s\n"), errstr);
     free(uc->transfer_file);
     if (waitpid(uc->pid, &status, 0) < 0)
-    	warn("Cannot wait for process - %s\n", errstr);
+    	warn(_("Cannot wait for process - %s\n"), errstr);
     else if (!WIFEXITED(status))
-    	warn("User process exited with status %d\n", status);
+    	warn(_("User process exited with status %d\n"), status);
     free(uc);
 }
 
@@ -1060,15 +1072,42 @@ user_conn_status_to_string(DCUserConn *uc, time_t now)
     char *status;
 
     if (uc->occupied_slot) {
-    	int percent = (uc->transfer_pos * 100)/max(uc->transfer_len, 1);
-	int rate = (now == 0 ? 0 : uc->transfer_pos/1024/max(now-uc->transfer_start, 1));
+    	int percent = (uc->transfer_pos * 100)/MAX(uc->transfer_len, 1);
+	int rate = (now == 0 ? 0 : uc->transfer_pos/1024/MAX(now-uc->transfer_start, 1));
 	char *file = base_name(uc->transfer_file);
-	char *dirstr = (uc->dir == DC_DIR_RECEIVE ? "Downloading" : "Uploading");
-
-        status = xasprintf("%s %3d%% (at %5d kb/s) %s", dirstr, percent, rate, file);
+	status = xasprintf(
+	    uc->dir == DC_DIR_RECEIVE
+		? _("Downloading %3d%% (at %5d kb/s) %s")
+		: _("Uploading %3d%% (at %5d kb/s) %s"),
+	    percent, rate, file);
     } else {
-	status = xasprintf("Idle");
+	status = xasprintf(_("Idle"));
     }
 
     return status;
 }
+
+bool
+parse_ip_and_port(char *source, struct sockaddr_in *addr, uint16_t defport)
+{
+    char *port;
+
+    port = strchr(source, ':');
+    if (port == NULL) {
+    	if (defport == 0)
+	    return false;
+	addr->sin_port = htons(defport);
+    } else {
+	*port = '\0';
+	port++;
+	if (!parse_uint16(port, &addr->sin_port))
+    	    return false;
+        addr->sin_port = htons(addr->sin_port);
+    }
+
+    if (!inet_aton(source, &addr->sin_addr))
+    	return false;
+
+    addr->sin_family = AF_INET;
+    return true;
+}
